<?php
/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://wpswings.com/
 * @since      1.0.0
 *
 * @package    Integration_With_Salesforce
 * @subpackage Integration_With_Salesforce/admin
 */

 use Automattic\WooCommerce\Internal\Admin\Orders\MetaBoxes\CustomMetaBox;

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Integration_With_Salesforce
 * @subpackage Integration_With_Salesforce/admin
 */
class Integration_With_Salesforce_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string $plugin_name       The name of this plugin.
	 * @param      string $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version     = $version;
	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Integration_With_Salesforce_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Integration_With_Salesforce_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		$screen = get_current_screen();
		$orderpage_screenid = Integration_With_Salesforce_Order_Handler::wps_get_shop_order_screen_id();

		if ( ! empty( $screen )
		&& ( in_array( $screen->id, $this->get_plugin_admin_screens() )
		|| ( ( $orderpage_screenid === $screen->id ) || ( 'product' === $screen->id ) ) ) ) {
			wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/integration-with-salesforce-admin.css', array(), time(), 'all' );
			wp_enqueue_style( $this->plugin_name . '-animate', plugin_dir_url( __FILE__ ) . 'css/integration-with-salesforce-animate.css', array(), time(), 'all' );
			wp_register_style( 'woocommerce_select2', WC()->plugin_url() . '/assets/css/select2.css', array(), WC_VERSION );

			wp_enqueue_style( 'woocommerce_select2' );
			wp_enqueue_style( 'woocommerce_admin_styles' );
		}
	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Integration_With_Salesforce_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Integration_With_Salesforce_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		$screen = get_current_screen();
		$orderpage_screenid = Integration_With_Salesforce_Order_Handler::wps_get_shop_order_screen_id();

		wp_enqueue_script( 'crm-connect-hubspot-sdk', '//js.hsforms.net/forms/shell.js', array(), $this->version, false );

		if ( ! empty( $screen ) && ( ( $orderpage_screenid === $screen->id ) || ( 'product' === $screen->id ) ) ) {

			wp_enqueue_script( $this->plugin_name . 'crm-connect-sweetalert', plugin_dir_url( __FILE__ ) . 'js/sweet-alert.js', array( 'jquery' ), $this->version, false );

			wp_enqueue_script( $this->plugin_name . 'crm-connnect-meta-box', plugin_dir_url( __FILE__ ) . 'js/order-meta-box-script.js', array( 'jquery' ), time(), false );
			$ajax_data = array(
				'crm'         => Integration_With_Salesforce::get_current_crm_name( 'slug' ),
				'ajax_url'    => admin_url( 'admin-ajax.php' ),
				'ajax_action' => 'mwb_woo_' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '_ajax',
				'ajax_nonce'  => wp_create_nonce( Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '_ajax_nonce' ),
				'screen_id'   => $screen->id,
				'order_page_screen_id' => $orderpage_screenid,
			);

			wp_localize_script( $this->plugin_name . 'crm-connnect-meta-box', 'ajax_data', $ajax_data );
		}

		if ( ! empty( $screen ) && in_array( $screen->id, $this->get_plugin_admin_screens() ) ) {

			wp_enqueue_script( $this->plugin_name . 'crm-connect-sweetalert', plugin_dir_url( __FILE__ ) . 'js/sweet-alert.js', array( 'jquery' ), $this->version, false );
			wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/integration-with-salesforce-admin.js', array( 'jquery', 'wc-enhanced-select', 'jquery-tiptip' ), $this->version, false );

			$ajax_data = array(
				'crm'                        => Integration_With_Salesforce::get_current_crm_name( 'slug' ),
				'ajax_url'                   => admin_url( 'admin-ajax.php' ),
				'ajax_action'                => 'mwb_woo_' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '_ajax',
				'ajax_nonce'                 => wp_create_nonce( Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '_ajax_nonce' ),
				'feed_back_link'             => admin_url( 'admin.php?page=integration-with-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '#mwb-feeds' ),
				'feed_back_text'             => __( 'Back to feeds', 'integration-with-salesforce' ),
				'delete_icon'                => INTEGRATION_WITH_SALESFORCE_URL . 'assets/src/images/trash.svg',
				'notice'                     => $this->get_admin_notice(),
				'empty_pricebook_message'    => __( 'Please enter pricebook name', 'integration-with-salesforce' ),
				'sync_complete_message'      => __( 'Congratulations!! Sync Process has been Completed!!', 'integration-with-salesforce' ),
				'trigger_error_message'      => __( 'Something Went Wrong', 'integration-with-salesforce' ),
				'process_complete_message'   => __( 'Process Completed', 'integration-with-salesforce' ),
				'fetching_feeds_message'     => __( 'Fetching default feeds data', 'integration-with-salesforce' ),
				'creating_feeds_message'     => __( 'Creating Feeds...', 'integration-with-salesforce' ),
				'access_token_renew_message' => __( 'Access token is renewed.', 'integration-with-salesforce' ),
				'select_pricebook_message'   => __( 'Please select a pricebook', 'integration-with-salesforce' ),
				'move_to_next_button_text'   => __( 'Move to Next', 'integration-with-salesforce' ),
				'required_field_message'     => __( 'Please enter the required fields', 'integration-with-salesforce' ),
				'connect'                    => array(
					'authorize' => __( 'Authorize', 'integration-with-salesforce' ),
					'login'     => __( 'Login & Authorize', 'integration-with-salesforce' ),
				),
				'sync_confirmation'          => array(
					'title'       => __( 'Are you sure?', 'integration-with-salesforce' ),
					'description' => __( 'It is recommended to check the fields mapping before syncing your data. For this you can skip this step by clicking on skip for now button at the bottom and check the fields mapping from feeds tab. You can sync the data using bulk sync after verifying the field mapping. Else, you can continue and start the syncing process by clicking on Sync Now.', 'integration-with-salesforce' ),
					'sync'        => __( 'Sync Now', 'integration-with-salesforce' ),
					'cancel'      => __( 'Cancel', 'integration-with-salesforce' ),
				),
				'disconnect'                 => array(
					'title'      => __( 'Are you sure?', 'integration-with-salesforce' ),
					'yes'        => __( 'Disconnect', 'integration-with-salesforce' ),
					'no'         => __( 'Cancel', 'integration-with-salesforce' ),
					'processing' => __( 'Disconnecting...', 'integration-with-salesforce' ),
				),
			);

			if ( 'mwb_' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '_feed' == $screen->post_type ) {
				$ajax_data['feed_form_settings'] = $this->get_feed_form_settings();
			}

			$this->clear_admin_notice();
			wp_localize_script( $this->plugin_name, 'ajax_data', $ajax_data );
		}
	}


	/**
	 * Check if mwb boilerplate is active.
	 *
	 * @param  string $submenu Submenu array.
	 * @return bool
	 */
	public function is_boilerplate_active( $submenu = '' ) {
		if ( ! empty( $submenu ) ) {
			return ( is_array( $submenu ) && ( array_key_exists( 'mwb-plugins', $submenu ) || array_key_exists( 'mwb-crm-plugins', $submenu ) ) );
		} else {
			return false;
		}
	}

	/**
	 * Include admin menu
	 */
	public function add_admin_menu_page() {

		global $submenu;

		if ( false == $this->is_boilerplate_active( $submenu ) ) { //phpcs:ignore

			add_menu_page(
				'WP Swings',
				'WP Swings',
				'manage_options',
				'mwb-crm-plugins',
				'',
				INTEGRATION_WITH_SALESFORCE_URL . '/assets/src/images/wpswings_logo.png',
				15
			);
			/**
			 * Filters the value of Salesforce plugin menu array.
			 *
			 * @since 1.0.3
			 *
			 * @param mixed  array.
			 */
			$wdc_menus = apply_filters( 'mwb_add_crm_plugins_menus_array', array() );
			if ( is_array( $wdc_menus ) && ! empty( $wdc_menus ) ) {
				foreach ( $wdc_menus as $wdc_key => $wdc_value ) {
					add_submenu_page( 'mwb-crm-plugins', $wdc_value['name'], $wdc_value['name'], 'manage_options', $wdc_value['menu_link'], array( $wdc_value['instance'], $wdc_value['function'] ) );
				}
			}
		}
	}

	/**
	 * WooCommerce Delivery Checker crm_admin_submenu_page.
	 *
	 * @since 1.0.0
	 * @param array $menus Marketplace menus.
	 */
	public function crm_admin_submenu_page( $menus = array() ) {

		$menus[] = array(
			'name'      => __( 'Salesforce Woo', 'integration-with-salesforce' ),
			'slug'      => 'integration-with-salesforce',
			'menu_link' => 'integration-with-salesforce',
			'instance'  => $this,
			'function'  => 'include_admin_menu_display',
		);

		return $menus;
	}

	/**
	 * Return salesforce sub-menu at custom post type page.
	 *
	 * @param mixed $submenu_file submenu_file.
	 * @param mixed $parent_file parent_file.
	 * @return mixed
	 */
	public function crm_bfw_set_submenu_file_to_handle_menu_for_wp_pages( $submenu_file, $parent_file ) {
		$mwb_bfw_taxonomy_array = 'mwb_salesforce_feed';
		if ( get_current_screen()->post_type == $mwb_bfw_taxonomy_array ) {
			$submenu_file = 'integration-with-salesforce';
		}

		return $submenu_file;
	}

	/**
	 * Return mwb-crm-plugins menu at custom post type page.
	 *
	 * @param mixed $parent_file parent_file.
	 * @return mixed
	 */
	public function prefix_highlight_taxonomy_parent_menu( $parent_file ) {
		global $submenu_file, $current_screen, $pagenow;
		$mwb_bfw_taxonomy_array = 'mwb_salesforce_feed';

		if ( get_current_screen()->post_type == 'mwb_salesforce_feed' ) {
			$parent_file = 'mwb-crm-plugins';
		}

		return $parent_file;
	}
	/**
	 * Removing default submenu of parent menu in backend dashboard
	 *
	 * @since   1.0.0
	 */
	public function remove_default_submenu() {
		global $submenu;
		if ( is_array( $submenu ) && array_key_exists( 'mwb-crm-plugins', $submenu ) ) {
			if ( isset( $submenu['mwb-crm-plugins'][0] ) ) {
				unset( $submenu['mwb-crm-plugins'][0] );
			}
		}
	}

	/**
	 * Ge plugin admin screens.
	 *
	 * @return array Array of admin screens.
	 */
	public function get_plugin_admin_screens() {

		return array( 'wp-swings_page_integration-with-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ), 'mwb_' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '_feed' );
	}

	/**
	 * Initiate all functionality of crm.
	 */
	public function woo_crm_init() {

		// Add authorisation screen.
		add_action( 'mwb-woo-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '-connect-authorisation-screen', array( $this, 'init_authorisation' ) );

		// Add dashbo screen.
		add_action( 'mwb-woo-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '-connect-dashboard-screen', array( $this, 'init_dashboard' ) );
	}

	/**
	 * Initiate authorisation screen.
	 */
	public function init_authorisation() {
		$file_path = 'admin/partials/templates/authorisation-screen.php';
		Integration_With_Salesforce_Framework::load_template( $file_path, array(), INTEGRATION_WITH_SALESFORCE_PATH );
	}

	/**
	 * Initiate all functionality of crm.
	 */
	public function init_dashboard() {
		$file_path = 'admin/partials/templates/dashboard-screen.php';
		Integration_With_Salesforce_Framework::load_template( $file_path, array(), INTEGRATION_WITH_SALESFORCE_PATH );
	}


	/**
	 * Include_settings_display function
	 */
	public function include_admin_menu_display() {
		$file_path = 'admin/partials/integration-with-salesforce-admin-display.php';
		Integration_With_Salesforce_Framework::load_template( $file_path, array(), INTEGRATION_WITH_SALESFORCE_PATH );
	}

	/**
	 * Get Tabs labels.
	 */
	public function render_tabs() {
		/**
		 * Filters the value of Salesforce tabs.
		 *
		 * @since 1.0.3
		 *
		 * @param mixed  array.
		 */
		$tabs      = apply_filters( 'mwb_crm_' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '_connect_tabs', array() );
		$file_path = 'woo-crm-fw/templates/crm-woo-tabs.php';
		Integration_With_Salesforce_Framework::load_template( $file_path, array( 'tabs' => $tabs ), INTEGRATION_WITH_SALESFORCE_PATH );
	}

	/**
	 * Get Tabs Contents.
	 */
	public function render_tab_contents() {
		/**
		 * Filters the value of Salesforce tabs contents.
		 *
		 * @since 1.0.3
		 *
		 * @param mixed  array.
		 */
		$tabs = apply_filters( 'mwb_crm_' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '_connect_tabs', array() );
		if ( ! empty( $tabs ) && is_array( $tabs ) ) {
			foreach ( $tabs as $file_name => $value ) {

				switch ( $file_name ) {
					case 'mwb-data-sync':
						$feed_module  = Integration_With_Salesforce_Feed_Module::get_instance();
						$feed_options = $feed_module->get_available_feeds_for_mapping();

						$settings[] = array(
							'title'       => esc_html__( 'Bulk Data Sync', 'integration-with-salesforce' ),
							'description' => esc_html__( 'Sync all the WooCommerce Objects of your site to the CRM in One Click.', 'integration-with-salesforce' ),
							'button'      => array(
								'title'  => esc_html__( 'Bulk Objects', 'integration-with-salesforce' ),
								'href'   => '',
								'newtab' => false,
								'class'  => 'bulk-data-sync',
							),
						);

						$params = compact( 'settings', 'feed_options' );
						break;

					case 'mwb-dashboard':
						$connection_status = self::get_connection_status();
						$issue_time        = self::get_access_token_issue_time();
						$instance_url      = self::get_connected_instance_url();

						$params = compact( 'connection_status', 'issue_time', 'instance_url' );
						break;

					case 'mwb-logs':
						$params = array();
						$options    = $this->get_supported_object_options( true );
						$params = compact( 'options' );
						break;

					case 'mwb-feeds':
						$args         = array( 'publish', 'draft' );
						$feed_module  = Integration_With_Salesforce_Feed_Module::get_instance();
						$feeds        = $feed_module->get_available_feeds( $args );
						$feed_options = $feed_module->get_feed_event_options();
						$params       = compact( 'feeds', 'feed_module', 'feed_options' );
						break;

					case 'mwb-advanced':
						$settings   = array();
						$options    = $this->get_supported_object_options();
						$settings[] = array(
							'title'         => esc_html__( 'Delete Salesforce Records', 'integration-with-salesforce' ),
							'description'   => esc_html__( 'Delete records from your salesforce account before resyncing data.', 'integration-with-salesforce' ),
							'select_id'     => 'mwb-crm-select-delete-object',
							'select_option' => $options,
							'button_id'     => 'mwb-delete-salesforce-record',
							'button_text'   => esc_html__( 'Delete Records', 'integration-with-salesforce' ),
						);

						$settings[] = array(
							'title'         => esc_html__( 'Reset Woo Meta Keys', 'integration-with-salesforce' ),
							'description'   => esc_html__( 'Delete meta keys from woocommerce in case you have deleted data from your salesforce account.', 'integration-with-salesforce' ),
							'select_id'     => 'mwb-crm-select-reset-meta',
							'select_option' => $options,
							'button_id'     => 'mwb-reset-woo-meta',
							'button_text'   => esc_html__( 'Delete Keys', 'integration-with-salesforce' ),
						);

						$params = compact( 'settings' );
						break;

					default:
						$params = array();
						break;
				}

				$file_path = 'woo-crm-fw/templates/tabs-contents/' . $file_name . '.php';
				Integration_With_Salesforce_Framework::load_template( $file_path, $params, INTEGRATION_WITH_SALESFORCE_PATH );
			}
		}
	}

	/**
	 * Specify Tabs Lables/Contents.
	 *
	 * @param array $tabs Array of plugin tabs.
	 */
	public function return_tabs( $tabs = array() ) {

		$tabs = array(
			'mwb-dashboard' => __( 'Dashboard', 'integration-with-salesforce' ),
			'mwb-feeds'     => __( 'Feeds', 'integration-with-salesforce' ),
			'mwb-data-sync' => __( 'Data Sync', 'integration-with-salesforce' ),
			'mwb-logs'      => __( 'Logs', 'integration-with-salesforce' ),
			'mwb-settings'  => __( 'Settings', 'integration-with-salesforce' ),
			'mwb-support'   => __( 'Support', 'integration-with-salesforce' ),
		);
		$secure_nonce      = wp_create_nonce( 'wps-salesfoce-nonce' );
		if ( INTEGRATION_WITH_SALESFORCE_DEBUG || ( isset( $_GET['show_dev'] ) && wp_verify_nonce( $secure_nonce, 'wps-salesfoce-nonce' ) ) ) {
			$tabs['mwb-advanced'] = __( 'Advanced', 'integration-with-salesforce' );
		}
		return $tabs;
	}

	/**
	 * Get connection status.
	 */
	public static function get_connection_status() {
		return get_option( 'mwb-woo-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '-authorised', false );
	}

	/**
	 * Get connect instance url.
	 */
	public function get_connected_instance_url() {
		$crm_class      = 'Integration_With_' . Integration_With_Salesforce::get_current_crm_name() . '_Api';
		$crm_api_module = $crm_class::get_instance();
		return $crm_api_module->get_instance_url();
	}

	/**
	 * Expiry for current Token.
	 */
	public static function get_access_token_issue_time() {
		$crm_class      = 'Integration_With_' . Integration_With_Salesforce::get_current_crm_name() . '_Api';
		$crm_api_module = $crm_class::get_instance();
		return $crm_api_module->get_access_token_issue_time();
	}

	/**
	 * Clear admin notices.
	 */
	public function clear_admin_notice() {
		delete_option( 'mwb-woo-salesforce-admin-notice' );
	}

	/**
	 * Get admin notices to show.
	 *
	 * @return array notice array.
	 */
	public function get_admin_notice() {
		return get_option( 'mwb-woo-salesforce-admin-notice', array() );
	}

	/**
	 * Add notice to options.
	 *
	 * @param string $message Notice error.
	 * @param bool   $success  success.
	 */
	public function add_admin_notice( $message, $success ) {
		update_option( 'mwb-woo-salesforce-admin-notice', compact( 'message', 'success' ) );
	}

	/**
	 * Get feed field types.
	 */
	public function get_feed_form_settings() {

		$class_name      = 'Integration_With_' . Integration_With_Salesforce::get_current_crm_name() . '_Framework';
		$class           = $class_name::get_instance();
		$options         = $class->getMappingDataset();
		$cpt_instance    = Integration_With_Salesforce_Feed_Module::get_instance();
		$available_feeds = $cpt_instance->get_available_feeds_for_mapping();
		if ( count( $available_feeds ) ) {
			$options['feeds'] = $available_feeds;
		}

		$settings['field_type'] = array(
			'label'   => __( 'Field Type', 'integration-with-salesforce' ),
			'options' => array(
				'standard_field' => __( 'Standard Fields', 'integration-with-salesforce' ),
				'custom_value'   => __( 'Custom Value', 'integration-with-salesforce' ),
			),
		);

		$settings['field_value'] = array(
			'label'   => __( 'Field Value', 'integration-with-salesforce' ),
			'options' => $options,
		);

		$settings['custom_value'] = array(
			'label' => __( 'Custom Value', 'integration-with-salesforce' ),
		);

		return $settings;
	}

	/**
	 * Perform Salesforce API Auth / Refresh Token.
	 *
	 * @since 1.0.0
	 */
	public function do_crm_auth() {

		if ( 'salesforce' != Integration_With_Salesforce::get_current_crm_name( 'slug' ) ) {
			return;
		}

		if ( ! empty( $_GET['perform-auth'] ) ) {

			if ( isset( $_GET['_wpnonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_GET['_wpnonce'] ) ) ) ) {

				$consumer_key    = ! empty( $_GET['consumer_key'] ) ? sanitize_text_field( wp_unslash( $_GET['consumer_key'] ) ) : false;
				$consumer_secret = ! empty( $_GET['consumer_secret'] ) ? sanitize_text_field( wp_unslash( $_GET['consumer_secret'] ) ) : false;
				$enviornment     = ! empty( $_GET['enviornment'] ) ? sanitize_text_field( wp_unslash( $_GET['enviornment'] ) ) : false;
				$use_custom_app  = ! empty( $_GET['use_custom_app'] ) ? sanitize_text_field( wp_unslash( $_GET['use_custom_app'] ) ) : 'no';

				if ( 'no' == $use_custom_app ) {
					$consumer_key    = '3MVG9fe4g9fhX0E7xz1Xm4Ewa5rcJI8I6Ga4rGD2SR2Fs4czhtbOsk.2yK652WN0r1F5ZBH65d_ZK41YHm4gT';
					$consumer_secret = 'E30C4E54EC11A4FA190C85CDD530EAEA9FEAB98063E55125A512AF4878714082';
				}

				if ( ! $consumer_key || ! $consumer_secret || ! $enviornment ) {
					return false;
				}

				update_option( 'mwb-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '-consumer-key', $consumer_key );
				update_option( 'mwb-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '-consumer-secret', $consumer_secret );
				update_option( 'mwb-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '-enviornment', $enviornment );
				update_option( 'mwb-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '-use-custom-app', $use_custom_app );
				$crm_api_module = Integration_With_Salesforce::get_integration_module( 'api' );
				$auth_url       = $crm_api_module->get_auth_code_url();
				wp_redirect( $auth_url ); //phpcs:ignore
				exit;
			}
		} elseif ( ! empty( $_GET['code'] ) ) {

			$crm_prefix = Integration_With_Salesforce::get_current_crm_name( 'slug' );

			if ( ! isset( $_GET['state'] ) || ! wp_verify_nonce( sanitize_text_field( wp_unslash( $_GET['state'] ) ), 'mwb_' . $crm_prefix . '_state' ) ) {
				wp_die( 'The state is not correct from Salesforce Server. Try again.' );
			}
			$auth_code      = ! empty( $_GET['code'] ) ? sanitize_text_field( wp_unslash( $_GET['code'] ) ) : false;
			$crm_api_module = Integration_With_Salesforce::get_integration_module( 'api' );
			$crm_api_module->save_access_token( $auth_code );
			wp_redirect( admin_url( 'admin.php?page=integration-with-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '&mwb_auth=1' ) ); //phpcs:ignore
			exit;

		} elseif ( ! empty( $_GET['perform-refresh'] ) ) {
			$crm_api_module = Integration_With_Salesforce::get_integration_module( 'api' );
			$crm_api_module->renew_access_token();
			wp_redirect( admin_url( 'admin.php?page=integration-with-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) ) ); //phpcs:ignore
			exit;

		} elseif ( ! empty( $_GET['perform-reauth'] ) ) {

			if ( isset( $_GET['_wpnonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_GET['_wpnonce'] ) ) ) ) {
				$crm_api_module = Integration_With_Salesforce::get_integration_module( 'api' );
				$auth_url       = $crm_api_module->get_auth_code_url();
				if ( ! $auth_url ) {
					return;
				}
				wp_redirect( $auth_url ); //phpcs:ignore
				exit;
			}
		} elseif ( ! empty( $_GET['mwb_download'] ) ) {

			if ( 'sync' == $_GET['mwb_download'] ) {
				$filename = WC_LOG_DIR . 'mwb-woo-salesforce-sync-log.log';
				header( 'Content-type: text/plain' );
				header( 'Content-Disposition: attachment; filename="' . basename( $filename ) . '"' );
				readfile( $filename ); //phpcs:ignore
				exit;
			}

			if ( 'api' == $_GET['mwb_download'] ) {
				$filename = WC_LOG_DIR . 'mwb-woo-salesforce-' . gmdate( 'Y-m-d' ) . '.log';
				if ( file_exists( $filename ) ) {
					header( 'Content-type: text/plain' );
					header( 'Content-Disposition: attachment; filename="' . basename( $filename ) . '"' );
					readfile( $filename ); //phpcs:ignore
					exit;
				}
			}
		}
	}

	/**
	 * Clear sync log callback.
	 */
	public function clear_sync_log() {
		$delete_duration  = get_option( 'mwb_salesforce_delete_log', 30 );
		$delete_timestamp = time() - ( $delete_duration * 24 * 60 * 60 );
		mwb_woo_salesforce_clear_log_data( $delete_timestamp );
	}

	/**
	 * Clear log table.
	 */
	public static function delete_sync_log() {
		global $wpdb;
		$table_name = $wpdb->prefix . 'mwb_woo_salesforce_log';
		$query      = "TRUNCATE TABLE {$table_name}";
		mwb_woo_salesforce_execute_db_query( $query );
	}

	/**
	 * Get supported object options for settings.
	 *
	 * @param mix $a If feed tiles are required.
	 * @return array Array of options.
	 */
	public function get_supported_object_options( $a = '' ) {

		$feed_module = Integration_With_Salesforce::get_integration_module( 'feed' );

		$all_feeds = $feed_module->get_available_feeds();

		if ( ! $all_feeds ) {
			return array();
		}
		$feed_list = array();
		foreach ( $all_feeds as $key => $feed ) {
			
			$crm_object = $feed_module->get_feed_data( $feed->ID, 'crm_object' );
			
			if ( ! empty( $a ) ) {
				$feed_list[ $crm_object ] = $feed->post_title;
			} else {
				$feed_list[ $crm_object ] = $crm_object;
			}

		}
		return $feed_list;
	}

	/**
	 * Fallback for account name field.
	 *
	 * @since 1.0.3
	 *
	 * @param Array $request Current request data.
	 * @param int   $obj_id  Woo object id.
	 * @param int   $feed_id Feed id.
	 * @return array Request data.
	 */
	public function filter_account_request_data( $request, $obj_id, $feed_id ) {

		if ( empty( $request['Name'] ) && ( get_post_meta( $feed_id, 'sync_person_account', true ) == 'no' ) ) {
			$request['Name'] = wps_salesforce_get_meta_data( $obj_id, '_billing_email', true );
		}

		return $request;
	}

	/**
	 * Check if log is enable.
	 *
	 * @return bool
	 */
	public static function is_background_sync_allowed() {
		$enable = get_option( 'mwb-salesforce-background-sync-allowed', 'no' );
		$enable = ( 'yes' === $enable );
		return $enable;
	}

	/**
	 * Add background sync schedule.
	 *
	 * @param array $schedules Array of available schedules.
	 * @return array
	 */
	public function add_background_sync_schedule( $schedules ) {
		if ( ! isset( $schedules['mwb_salesforce_bg_sync'] ) ) {

			$schedules['mwb_salesforce_bg_sync'] = array(
				'interval' => 5 * 60,
				'display'  => __( 'Once every 5 minutes', 'integration-with-salesforce' ),
			);

			if ( ! isset( $schedules['mwb_woo_sf_product_fetch'] ) ) {
				$schedules['mwb_woo_sf_product_fetch'] = array(
					'interval' => 5 * 60,
					'display'  => __( 'Once every 5 minutes', 'integration-with-salesforce' ),
				);
			}
		}

		return $schedules;
	}

	/**
	 * Schedule background sync event.
	 */
	public function schedule_background_sync_event() {

		// Scheduled event for background syncing.
		if ( ! wp_next_scheduled( 'mwb_salesforce_background_sync' ) ) {
			wp_schedule_event( time(), 'mwb_salesforce_bg_sync', 'mwb_salesforce_background_sync' );
		}


		if ( ! wp_next_scheduled( 'mwb_woo_salesforce_product_fetch' ) ) {
			wp_schedule_event( time(), 'mwb_woo_sf_product_fetch', 'mwb_woo_salesforce_product_fetch' );
		}
	}

	/**
	 * Perform background sync.
	 */
	public function perform_background_sync() {

		$background_sync = self::is_background_sync_allowed();
		$connected       = self::get_connection_status();

		if ( $connected && $background_sync ) {

			$framework        = Integration_With_Salesforce::get_integration_module( 'framework-base' );
			$all_feeds        = $framework->get_all_feed();
			$default_feeds    = array();
			$additional_feeds = array();
			$product_feeds    = array();
			$users_feeds    = array();
			$order_feeds      = array();
			$request_module   = Integration_With_Salesforce::get_integration_module( 'request' );
			if ( ! empty( $all_feeds ) && is_array( $all_feeds ) ) {
				foreach ( $all_feeds as $key => $feed_id ) {
					$is_default_feed = $framework->get_feed( $feed_id, 'default_feed' );
					if ( $is_default_feed ) {
						array_push( $default_feeds, $feed_id );
					} else {
						array_push( $additional_feeds, $feed_id );
					}
				}
			}
			if ( ! empty( $default_feeds ) ) {
				foreach ( $default_feeds as $key => $feed_id ) {
					$record_type = $framework->get_feed( $feed_id, 'crm_object' );
					if ( 'Product2' === $record_type ) {
						$product_feeds['0'] = $feed_id;
					} elseif ( 'PricebookEntry' === $record_type ) {
						$product_feeds['1'] = $feed_id;
					} elseif ( 'Account' === $record_type ) {
						$order_feeds['0'] = $feed_id;
					} elseif ( 'Contact' === $record_type ) {
						$order_feeds['1'] = $feed_id;
					} elseif ( 'Opportunity' === $record_type ) {
						$order_feeds['2'] = $feed_id;
					} elseif ( 'Order' === $record_type ) {
						$order_feeds['3'] = $feed_id;
					} elseif ( 'OrderItem' === $record_type ) {
						$order_feeds['4'] = $feed_id;
					}
				}
			}
			if ( ! empty( $additional_feeds ) ) {
				foreach ( $additional_feeds as $key => $feed_id ) {
					$record_type = $framework->get_feed( $feed_id, 'crm_object' );
					$woo_object  = $request_module->get_feed_woo_object( $record_type, $feed_id );
					if ( 'users' === $woo_object ) {
						array_push( $users_feeds, $feed_id );
					} elseif ( 'product' === $woo_object ) {
						array_push( $product_feeds, $feed_id );
					} else {
						array_push( $order_feeds, $feed_id );
					}
				}
			}

			ksort( $product_feeds );
			ksort( $order_feeds );
			ksort( $users_feeds );

			$product_args = array(
				'post_type'      => 'product',
				'posts_per_page' => 1,
				'post_status'    => 'all',
				'meta_query'     => array(  //phpcs:ignore
					'relation' => 'OR',
					array(
						'key'     => 'mwb_salesforce_allow_background_syncing',
						'compare' => 'NOT EXISTS',
					),
					array(
						'key'     => 'mwb_salesforce_allow_background_syncing',
						'compare' => '=',
						'value'   => 'yes',
					),
				),
			);

			$product_query = new WP_Query( $product_args );
			while ( $product_query->have_posts() ) :
				$product_query->the_post();
				$prod_woo_id = get_the_ID();
				break;
			endwhile;

			if ( ! empty( $product_feeds ) && ! empty( $prod_woo_id ) ) {
				foreach ( $product_feeds as $key => $product_feed_id ) {
					$request_module->trigger_product_related_feed( $product_feed_id, $prod_woo_id, 'true' );
					update_post_meta( $prod_woo_id, 'mwb_salesforce_allow_background_syncing', 'no' );
				}
			}

			if ( Integration_With_Salesforce_Order_Handler::wps_get_cot_enabled_order_enabled() ) {
				$order_query = new WC_Order_Query( array(
					'status' => 'all',
					'orderby' => 'date',
					'order' => 'DESC',
					'return' => 'ids',
					'limit'  => -1,
					'meta_query' =>array( //phpcs:ignore
						'relation' => 'OR',
						array(
							'key'     => 'mwb_salesforce_allow_background_syncing',
							'compare' => 'NOT EXISTS',
						),
						array(
							'key'     => 'mwb_salesforce_allow_background_syncing',
							'compare' => '=',
							'value'   => 'yes',
						),
					),
				) );
				foreach ( $order_query->get_orders() as $key => $value ) {
					$woo_id = $value;
					break;
				}
			} else {

				$args = array(
					'post_type'      => 'shop_order',
					'posts_per_page' => 1,
					'post_status'    => 'any',
					'limit'  => -1,
					'meta_query'     => array(  //phpcs:ignore
						'relation' => 'OR',
						array(
							'key'     => 'mwb_salesforce_allow_background_syncing',
							'compare' => 'NOT EXISTS',
						),
						array(
							'key'     => 'mwb_salesforce_allow_background_syncing',
							'compare' => '=',
							'value'   => 'yes',
						),
					),
				);

				$order_query = new WP_Query( $args );
				while ( $order_query->have_posts() ) :
					$order_query->the_post();
					$woo_id = get_the_ID();
					break;
				endwhile;
			}

			if ( ! empty( $order_feeds ) && ! empty( $woo_id ) ) {
				foreach ( $order_feeds as $key => $order_feed_id ) {
					$request_module->perform_shop_order_sync( $order_feed_id, $woo_id );
					if ( Integration_With_Salesforce_Order_Handler::wps_get_cot_enabled_order_enabled() ) {
						$order = wc_get_order($woo_id);
						if ( $order ) {

							$order->update_meta_data( 'mwb_salesforce_allow_background_syncing', 'no' );
							$order->save();
						} else {
							update_post_meta( $woo_id, 'mwb_salesforce_allow_background_syncing', 'no' );
						}
					} else {
						update_post_meta( $woo_id, 'mwb_salesforce_allow_background_syncing', 'no' );
					}
				}
			}
			$user_args = array(
				'number'     => 1,
				'meta_query' => array( //phpcs:ignore
					'relation' => 'OR',
					array(
						'key'     => 'wps_salesforce_allow_background_syncing',
						'value'   => 'yes',
						'compare' => '=',
					),
					array(
						'key'     => 'wps_salesforce_allow_background_syncing',
						'compare' => 'NOT EXISTS',
					),
				),
			);

			$user_query = new WP_User_Query( $user_args );

			// Get the user.
			$users = $user_query->get_results();
			if ( ! empty( $users ) ) {
				foreach ( $users as $user ) {
					$woo_user_id = $user->ID;
				}
			}

			if ( ! empty( $user_feeds ) && ! empty( $woo_user_id ) ) {
				foreach ( $user_feeds as $key => $user_feed_id ) {
					$request_module->perform_wp_user_sync( $user_feed_id, $woo_user_id );
					update_user_meta( $woo_user_id, 'wps_salesforce_allow_background_syncing', 'no' );
				}
			}

		}


		if ( 'yes' == get_option( 'mwb-salesforce-enable-product-deletion-salesforce-to-woo', 'no' ) ) {
			$product_args = array(
				'post_type'      => 'product',
				'posts_per_page' => -1,
				'post_status'    => 'all',
				'meta_query'     => array(  //phpcs:ignore
					array(
						'key'     => 'mwb_salesforce_Product2_id',
						'compare' => 'EXISTS',
					),
				),
			);
	
			$product_query = new WP_Query( $product_args );
			while ( $product_query->have_posts() ) :
				$product_query->the_post();
				$prd_id[] = get_the_ID();
				$salesforce_prod_id = get_post_meta( get_the_ID(), 'mwb_salesforce_Product2_id', true );
				if ( !empty( $salesforce_prod_id ) ) {
					$this->delete_trash_or_restore_product_from_wordpress( $salesforce_prod_id, get_the_ID() );
				}
			endwhile;
		}
	}

	/**
	 * Check if instant sync is enable.
	 *
	 * @return bool
	 */
	public static function is_wipe_allowed() {
		$enable = get_option( 'mwb-salesforce-wipe-data', 'no' );
		$enable = ( 'yes' === $enable );
		return $enable;
	}

	/**
	 * Fallback for lead name field.
	 *
	 * @since 1.0.3
	 *
	 * @param Array $request Current request data.
	 * @param int   $obj_id  Woo object id.
	 * @param int   $feed_id Feed id.
	 * @return array Request data.
	 */
	public function filter_lead_request_data( $request, $obj_id, $feed_id ) {

		if ( isset( $request['Name'] ) ) {
			unset( $request['Name'] );
		}

		return $request;
	}

	/**
	 * Function to add membership options in order values dropdown.
	 *
	 * @param array $order_callbacks add membership options in order values dropdown.
	 * @return void
	 */
	public function woo_salesforce_order_callback_fn( $order_callbacks ) {

		if ( $this->wps_is_woocommerce_membership_plugin_active() ) {
			$order_call_membership = array(
				'membership_plan_id'      => 'get_membership_plan_id',
				'membership_created_time' => 'get_membership_create_time',
				'membership_create_date'  => 'get_membership_create_date',
				'membership_end_date'     => 'get_membership_end_date',
				'membership_end_time'     => 'get_membership_end_time',
				'membership_next_bill_on' => 'get_membership_next_bill_on',
				'membership_subscription' => 'get_membership_subscription',
				'membership_order_total' => 'get_membership_order_total',
				'membership_order_date' => 'get_membership_order_date',
				'membership_purchased_in' => 'get_membership_purchased_in',
				'membership_expiry' => 'get_membership_expiry',
				'membership_since' => 'get_membership_since',
				'membership_product_name' => 'get_membership_product_name',
				'membership_product_id' => 'get_membership_product_id',
				'membership_status' => 'get_membership_status',
				'membership_user_id' => 'get_membership_user_id',
				'user_membership_id' => 'get_user_membership_id',
				'membership_plan'   => 'get_membership_plan',
			);
			$order_callbacks = array_merge( $order_callbacks, $order_call_membership );
		}

		if ( $this->wps_is_woocommerce_subscription_plugin_active() ) {
			$order_call_subscription = array(
				'subscription_id'       => 'get_subscription_id',
				'subscription_status'   => 'get_subscription_status',
				'subscription_customer'   => 'get_subscription_customer',
				'subscription_payment_method'   => 'get_subscription_payment_method',
				'subscription_payment_schedule'   => 'get_subscription_payment_schedule',
				'subscription_start_date'   => 'get_subscription_start_date',
				'subscription_end_date'   => 'get_subscription_end_date',
				'subscription_next_payment' => 'get_subscription_next_payment',
				'subscription_information'  => 'get_subscription_information',
				'subscription_variation_id' => 'get_subscription_variation_id',
				'subscription_item_name'  => 'get_subscription_item_name',
				'subscription_item_price'  => 'get_subscription_item_price',
			);
			$order_callbacks = array_merge( $order_callbacks, $order_call_subscription );
		}

		return $order_callbacks;
	}

	/**
	 * Function to check for plugin activation.
	 *
	 * @return   bool
	 */
	public static function wps_is_woocommerce_membership_plugin_active() {

		$slug = 'woocommerce-memberships/woocommerce-memberships.php';

		$active_plugins = get_option( 'active_plugins', array() );

		if ( is_multisite() ) {
			$active_plugins = array_merge( $active_plugins, get_option( 'active_sitewide_plugins', array() ) );
		}

		return in_array( $slug, $active_plugins, true ) || array_key_exists( $slug, $active_plugins );
	}

	/**
	 * Function to check for plugin activation.
	 *
	 * @return   bool
	 */
	public static function wps_is_woocommerce_subscription_plugin_active() {

		$slug = 'woocommerce-subscriptions/woocommerce-subscriptions.php';

		$active_plugins = get_option( 'active_plugins', array() );

		if ( is_multisite() ) {
			$active_plugins = array_merge( $active_plugins, get_option( 'active_sitewide_plugins', array() ) );
		}

		return in_array( $slug, $active_plugins, true ) || array_key_exists( $slug, $active_plugins );
	}

	/**
	 * Fetch products from Salesforce crm.
	 */
	public function perform_sf_product_fetch() {

		$update_qty = get_option('mwb-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '-update-product-stock', 'no' );
		$connected       = self::get_connection_status();

		if ( 'yes' != $update_qty ) {
			return;
		}

		$product_stock_field_api_name = get_option( 'mwb-' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '-update-product-stock-api-name', '' );
		if ( empty( $product_stock_field_api_name ) ) {
			return;
		}

		$query = sprintf( 'SELECT Id,Name,%s FROM Product2', $product_stock_field_api_name );
		
		$crm_class      = 'Integration_With_' . Integration_With_Salesforce::get_current_crm_name() . '_Api';
		$crm_api_module = $crm_class::get_instance();
		$salesforce_products= $crm_api_module->query_records( $query );

		if ( ! $connected && empty( $salesforce_products ) ) {
			return;
		}

		foreach ( $salesforce_products as $key => $salesforce_product ) {
			$salesforce_id = ! empty( $salesforce_product['Id'] ) ? $salesforce_product['Id'] : 0;

			if ( $salesforce_id ) {
				$woo_product_id = $this->get_woo_product_id_by_salesforce_id( $salesforce_id );
				
				if ( $woo_product_id ) {
					$woo_product = wc_get_product( $woo_product_id );
					if ( $woo_product ) {

						$product_stock = $woo_product->get_stock_quantity();
						$salesforce_stock    = $salesforce_product[$product_stock_field_api_name];

						if ( $product_stock != $salesforce_stock ) {
							wc_update_product_stock( $woo_product, $salesforce_stock );

							$log_args = array(
								'event'         => 'update_stock',
								'woo_id'        => $woo_product_id,
								'salesforce_id' => $salesforce_id,
								'product_stock' => $product_stock,
								'salesforce_stock'    => $salesforce_stock,
							);

							$this->create_sync_log_entry( $log_args );
						}
					}
				}
			}
		}
	}

	/**
	 * Get woo product id by Salesforce product id.
	 *
	 * @param  string $salesforce_id Salesforce record id.
	 * @return int|bool        Woo product id.
	 */
	public function get_woo_product_id_by_salesforce_id( $salesforce_id ) {

		$product_types = array(
			'product',
			'product_variation',
		);

		$woo_product_id = false;

		$args = array(
			'post_type'   => $product_types,
			'post_status' => 'any',
			'meta_query'     => array(  //phpcs:ignore,
				array(
					'key'     => 'mwb_salesforce_Product2_id',
					'compare' => '=',
					'value'   => $salesforce_id,
				),
			),
		);

		$product_post = get_posts( $args );

		if ( ! empty( $product_post ) && ! empty( $product_post[0] ) ) {
			$woo_product_id = $product_post[0]->ID;
		}

		return $woo_product_id;
	}

	/**
	 * Create log for product quantity and order status.
	 *
	 * @param array $log_args log data.
	 * @return void
	 */
	public function create_sync_log_entry( $log_args ) {

		if ( ! Integration_With_Salesforce_Framework::is_log_enable() ) {
			return;
		}

		$log_dir = WC_LOG_DIR . 'mwb-woo-sf-fetch-' . gmdate( 'Y-m-d' ) . '.log';

		if ( ! is_dir( $log_dir ) ) {
			@fopen( WC_LOG_DIR . 'mwb-woo-sf-fetch-' . gmdate( 'Y-m-d' ) . '.log', 'a' ); //phpcs:ignore
		}

		global $wp_filesystem;  // Define global object of WordPress filesystem.
		WP_Filesystem();

		$file_data = '';
		if ( file_exists( $log_dir ) ) {
			$file_data = $wp_filesystem->get_contents( $log_dir );
		} else {
			$file_datforcea = '';
		}

		$log = '------------------------------------' . PHP_EOL;

		foreach ( $log_args as $l_key => $l_value ) {
			$log .= strtoupper( $l_key ) . ' : ' . $l_value . PHP_EOL;
		}

		$log .= 'Time: ' . current_time( 'F j, Y  g:i a' ) . PHP_EOL;
		$log .= '------------------------------------' . PHP_EOL;

		$file_data .= $log;
		$wp_filesystem->put_contents( $log_dir, $file_data );
	}
	/**
	 * Add dummy edit user profile hook to manage param count.
	 *
	 * @param int $user_id Id of updated user.
	 *
	 * @since 1.0.2
	 */
	public function add_wps_sf_edit_user_profile_update_hook( $user_id ) {
		/**
		 * Trigger associated Feeds.
		 *
		 * @since 1.0.0
		 *
		 * @param mixed  $user_id  Woo user id.
		 * @param mixed array.
		 */
		do_action( 'wps_sf_edit_user_profile_update', $user_id, array() );
	}

	
	/**
	 * Filter contact request to check empty values.
	 *
	 * @param array  $request Request data.
	 * @param array  $user_id Woo order id.
	 * @param string $record_type record type.
	 * @return array.
	 */
	public function wps_salesforce_get_filtered_user_request( $request, $user_id, $record_type = '' ) {

		$user = get_user_by( 'id', $user_id );

		if ( $user && ! empty( $user ) ) {

			$request['Last_Name'] = empty( $request['Last_Name'] ) ? $user->user_nicename : $request['Last_Name'];

			$request['First_Name'] = empty( $request['First_Name'] ) ? $user->user_nicename : $request['First_Name'];

			if ( isset( $request['Full_Name'] ) && empty( trim( $request['Full_Name'] ) ) ) {
				$request['Full_Name'] = $user->user_nicename . ' ' . $user->user_nicename;
			}

			$request['Email'] = empty( $request['Email'] ) ? $user->user_email : $request['Email'];
		}

		return $request;
	}

	/**
	 * Fallback for account name field.
	 *
	 * @since 1.0.3
	 *
	 * @param Array $request Current request data.
	 * @param int   $obj_id  Woo object id.
	 * @param int   $feed_id Feed id.
	 * @return array Request data.
	 */
	public function filter_account_request_data_for_user( $request, $user_id, $feed_id ) {
		$user = get_user_by( 'id', $user_id );
		if ( empty( $request['Name'] ) && ( get_post_meta( $feed_id, 'sync_person_account', true ) == 'no' ) ) {
			$request['Name'] = empty( $request['Email'] ) ? $user->user_email : $request['Email'];
		}

		return $request;
	}


	public function action_wp_trash_post( $post_id ) {

		if (get_post_type($post_id) === 'product') {

			$framework        = Integration_With_Salesforce::get_integration_module( 'framework-base' );
			$all_feeds        = $framework->get_all_feed();
			$default_feeds    = array();
			$additional_feeds = array();
			$product_feeds    = array();
			$request_module   = Integration_With_Salesforce::get_integration_module( 'request' );
			$feed_module = Integration_With_Salesforce::get_integration_module( 'feed' );
			if ( ! empty( $all_feeds ) && is_array( $all_feeds ) ) {
				foreach ( $all_feeds as $key => $feed_id ) {
					$is_default_feed = $framework->get_feed( $feed_id, 'default_feed' );
					if ( $is_default_feed ) {
						array_push( $default_feeds, $feed_id );
					} else {
						array_push( $additional_feeds, $feed_id );
					}
				}
			}
			if ( ! empty( $default_feeds ) ) {
				foreach ( $default_feeds as $key => $feed_id ) {
					$record_type = $framework->get_feed( $feed_id, 'crm_object' );
					if ( 'Product2' === $record_type ) {
						$product_feeds['0'] = $feed_id;
					}
				}
			}
			if ( ! empty( $additional_feeds ) ) {
				foreach ( $additional_feeds as $key => $feed_id ) {
					$record_type = $framework->get_feed( $feed_id, 'crm_object' );
					$woo_object  = $request_module->get_feed_woo_object( $record_type, $feed_id );
					if ( 'product' === $woo_object ) {
						array_push( $product_feeds, $feed_id );
					}
				}
			}

			ksort( $product_feeds );

			$feed_id = $product_feeds[0];

			if ( ! empty( $feed_id ) && is_numeric( $feed_id ) ) {

				$request_module->trigger_product_related_feed( $feed_id, $post_id, 'false' );

			}

		}
	}

	public function action_wp_untrash_post( $post_id ) {

		if (get_post_type($post_id) === 'product') {

			$framework        = Integration_With_Salesforce::get_integration_module( 'framework-base' );
			$all_feeds        = $framework->get_all_feed();
			$default_feeds    = array();
			$additional_feeds = array();
			$product_feeds    = array();
			$request_module   = Integration_With_Salesforce::get_integration_module( 'request' );
			$feed_module = Integration_With_Salesforce::get_integration_module( 'feed' );
			if ( ! empty( $all_feeds ) && is_array( $all_feeds ) ) {
				foreach ( $all_feeds as $key => $feed_id ) {
					$is_default_feed = $framework->get_feed( $feed_id, 'default_feed' );
					if ( $is_default_feed ) {
						array_push( $default_feeds, $feed_id );
					} else {
						array_push( $additional_feeds, $feed_id );
					}
				}
			}
			if ( ! empty( $default_feeds ) ) {
				foreach ( $default_feeds as $key => $feed_id ) {
					$record_type = $framework->get_feed( $feed_id, 'crm_object' );
					if ( 'Product2' === $record_type ) {
						$product_feeds['0'] = $feed_id;
					}
				}
			}
			if ( ! empty( $additional_feeds ) ) {
				foreach ( $additional_feeds as $key => $feed_id ) {
					$record_type = $framework->get_feed( $feed_id, 'crm_object' );
					$woo_object  = $request_module->get_feed_woo_object( $record_type, $feed_id );
					if ( 'product' === $woo_object ) {
						array_push( $product_feeds, $feed_id );
					}
				}
			}

			ksort( $product_feeds );

			$feed_id = $product_feeds[0];

			if ( ! empty( $feed_id ) && is_numeric( $feed_id ) ) {

				$request_module->trigger_product_related_feed( $feed_id, $post_id, 'true' );

			}

		}
	}

	public function custom_function_on_product_delete( $post_id ) {
		// Check if the deleted post is a product
		if (get_post_type($post_id) === 'product') {
			$record_id = get_post_meta( $post_id, 'mwb_salesforce_Product2_id', true );
			$api_class = new Integration_With_Salesforce_Api();
			$response = $api_class->delete_record( $record_id, 'Product2', array() );
		}
	}

	public function delete_trash_or_restore_product_from_wordpress( $salesforce_prod_id, $woo_prod_id ) {
		$api_class = new Integration_With_Salesforce_Api();
		$response = $api_class->get_record( $salesforce_prod_id, 'Product2', array() );
		if ( isset( $response['code'] ) && '200' == $response['code'] && !empty( $response['data'] ) ) {
			$isActive = $response['data']['IsActive'];
			if ( ! $isActive ) {
				if ( 'publish' == wc_get_product($woo_prod_id)->get_data()['status'] ) {
					wp_trash_post( $woo_prod_id );
				}
			} elseif ( 'trash' == wc_get_product($woo_prod_id)->get_data()['status'] ) {
					wp_publish_post( $woo_prod_id );
			}
		}
		if ( !empty( $response['data'][0] ) ) {
			if ( ( isset( $response['code'] ) && '404' == $response['code'] ) || ( isset( $response['code'] ) && 'NOT_FOUND' == $response['data'][0]['errorCode'] ) ) {
				wp_delete_post( $woo_prod_id );
			}
		}
	}

	// End of class.
}
