<?php
/**
 * Provide a admin area view for the plugin
 *
 * This file is used to markup the admin-facing aspects of the plugin.
 *
 * @link       https://wpswings.com/
 * @since      1.0.0
 *
 * @package    Integration_With_Salesforce
 * @subpackage Integration_With_Salesforce/admin/partials
 */

$authorised         = get_option( 'mwb-woo-salesforce-authorised', false );
$intro_hide         = $authorised ? 'section-hide' : '';
$form_hide          = $authorised ? '' : 'section-hide';
$pricebook_options  = get_option( 'mwb_salesforce_pricebook_data', array() );
$selected_pricebook = get_option( 'mwb_salesforce_selected_pricebook', '' );

?>
<div class="mwb-progress__wrap">
	<div class="mwb-progress">
	<div class="mwb-progress__bar step-1 active">.</div>
  </div>
  <div class="mwb-progress__step-wrap">
	<div class="mwb-progress__step">

	</div>
	<div class="mwb-progress__step">
		<?php esc_html_e( 'Setup Connection', 'integration-with-salesforce' ); ?>
	</div>
	<div class="mwb-progress__step">
		<?php esc_html_e( 'Setup Pricebook', 'integration-with-salesforce' ); ?>
	</div>
	<div class="mwb-progress__step">
		<?php esc_html_e( 'Create Feeds', 'integration-with-salesforce' ); ?>
	</div>
	<div class="mwb-progress__step">
		<?php esc_html_e( 'Completed', 'integration-with-salesforce' ); ?>
	</div>
</div>
</div>
<section class="mwb-intro">
	<div class="mwb-content-wrap">
		<div class="mwb-intro__header">
			<h2 class="mwb-section__heading">
				<?php
				// translators: CRM Name.
				echo esc_html( sprintf( __( 'Getting started with %s and WooCommerce.', 'integration-with-salesforce' ), Integration_With_Salesforce::get_current_crm_name() ) );
				?>
			</h2>
		</div>
		<?php if ( '1' !== get_option( 'mwb-woo-salesforce-authorised', false ) || '1' !== get_option( 'mwb_salesforce_valid_api', false ) ) : ?>
		<div class="mwb-intro__body mwb-intro__content <?php echo esc_attr( $intro_hide ); ?>">
			<p>
			<?php
			echo esc_html(
				sprintf(
					/* translators: 1: CRM Name 2: CRM Name 3: CRM Name 4: CRM Name */
					__(
						'With this WooCommerce %1$s Integration you can easily sync all your WooCommerce data over %2$s CRM. It will create Accounts, Contacts, Opportunities and Orders over %3$s CRM, based on your WooCommerce Order data.It will also create Products over %4$s CRM and map them with Order.',
						'integration-with-salesforce'
					),
					Integration_With_Salesforce::get_current_crm_name(),
					Integration_With_Salesforce::get_current_crm_name(),
					Integration_With_Salesforce::get_current_crm_name(),
					Integration_With_Salesforce::get_current_crm_name()
				)
			);
			?>
			</p>
			<ul class="mwb-intro__list">
				<li class="mwb-intro__list-item">
					<?php
					// translators: CRM Name.
					echo esc_html( sprintf( __( 'Connect your %s CRM account with WooCommerce.', 'integration-with-salesforce' ), Integration_With_Salesforce::get_current_crm_name() ) );
					?>
				</li>
				<li class="mwb-intro__list-item">
					<?php
					esc_html_e(
						'Setup Pricebook to Sync your WooCommerce Products.',
						'integration-with-salesforce'
					);
					?>
				</li>
				<li class="mwb-intro__list-item">
				<?php esc_html_e( 'Create feeds for Accounts, Contacts, Products, Opportunities and Orders.', 'integration-with-salesforce' ); ?>
				</li>
				<li class="mwb-intro__list-item">
				<?php
				// translators: CRM Name.
				echo esc_html( sprintf( __( 'Sync your data over %s CRM.', 'integration-with-salesforce' ), Integration_With_Salesforce::get_current_crm_name() ) );
				?>
				</li>
			</ul>
			<div class="mwb-intro__button">
				<a href="#" class="mwb-btn mwb-btn--filled" onclick="showAuthForm( event )">
					<?php
					esc_html_e(
						'Connect your Account',
						'integration-with-salesforce'
					);
					?>
				</a>
			</div>
		</div>  
		<div class="mwb-intro__body mwb-salesforce-auth-wrap <?php echo esc_attr( $form_hide ); ?>">
			<?php
			$crm_admin = 'Integration_With_Salesforce_Framework';
			$settings  = $crm_admin::get_authorization_settings();
			?>
			<div class="mwb-auth-notice-wrap hide-notice" >
				<p class="mwb-auth-notice-text">
				<?php
				esc_html_e(
					'Authorization has been successful ! Validating Api details .....',
					'integration-with-salesforce'
				);
				?>
				</p>
			</div>
			<form action="#" method="post" id="mwb-salesforce-auth-portal">
				<?php woocommerce_admin_fields( $settings ); ?>
			</form>
			<div class="mwb-intro__button">
				<a href="<?php echo esc_url( wp_nonce_url( admin_url( '?perform-auth=1' ) ) ); ?>" class="mwb-btn mwb-btn--filled" id="mwb-<?php echo esc_html( Integration_With_Salesforce::get_current_crm_name( 'slug' ) ); ?>-authorize-button" >
					<?php esc_html_e( 'Authorize', 'integration-with-salesforce' ); ?>
				</a>
				<div class="mwb-intro__bottom-text-wrap">
					<?php
					esc_html_e(
						'Check app setup guide.',
						'
                    integration-with-salesforce'
					);
					?>
					<a href="javascript:void(0)" class="mwb-btn__bottom-text trigger-setup-guide"><?php esc_html_e( 'Click Here', 'integration-with-salesforce' ); ?></a></p>
				</div>
			</div>
		</div>
		<?php else : ?>
		<div class="mwb-intro__body mwb-salesforce-auth-success">
			<p>
				<?php esc_html_e( 'Authentication and Api validation has been successful, you are all set to proceed to the next step.', 'integration-with-salesforce' ); ?>  
			</p>
			<div class="mwb-intro__button">
				<a href="#" class="mwb-btn mwb-btn--filled progress-next show-pricebook-btn">
					<?php esc_html_e( 'Move to next step', 'integration-with-salesforce' ); ?>
				</a>
			</div>
		</div>
		<?php endif; ?>
	</div>
</section>
<section class="mwb-pricebook">
	<div class="mwb-content-wrap">
		<div class="mwb-properties__header">
			<h2 class="mwb-section__heading">
				<?php esc_html_e( 'Setup Pricebook', 'integration-with-salesforce' ); ?>
			</h2>
		</div>
		<div class="mwb-properties__body mwb-properties__content">
			<p class="mwb-properties__text">
				<?php
				esc_html_e( 'Please Select a pricebook or create a new one to map your WooCommerce Products over Salesforce.', 'integration-with-salesforce' );
				?>
			</p>
		</div>
		<div class="mwb-properties__body mwb-properties__pricebook-form select-pricebook-form">
			<select id="pricebook-select" name="selected_pricebook">
			<option value=""><?php esc_html_e( 'Select a pricebook', 'integration-with-salesforce' ); ?></option>
			<?php if ( count( $pricebook_options ) ) : ?>
				<?php foreach ( $pricebook_options as $key => $pricebook ) : ?>
					<option value="<?php echo esc_attr( $pricebook['Id'] ); ?>"
						<?php echo esc_attr( selected( $selected_pricebook, $pricebook['Id'] ) ); ?>>
						<?php echo esc_html( $pricebook['Name'] ); ?>
					</option>
				<?php endforeach; ?>
			<?php endif; ?>
			</select>
			<a id="mwb-woo-salesforce-refresh-pricebook" 
				class="mwb-btn mwb-btn--filled refresh-pricebook">
				<?php esc_html_e( 'Refresh Pricebooks', 'integration-with-salesforce' ); ?>
			</a>
		</div>
		<div class="mwb-properties__body mwb-properties__pricebook-form select-pricebook-form">
			<a id="mwb-woo-salesforce-show-pricebook-form" 
				class="mwb-btn mwb-btn--filled add-pricebook">
				<?php esc_html_e( 'Create New', 'integration-with-salesforce' ); ?>
			</a>
			<a id="mwb-woo-salesforce-save-pricebook" 
				class="mwb-btn mwb-btn--filled add-pricebook">
				<?php esc_html_e( 'Select and move to next', 'integration-with-salesforce' ); ?>
			</a>
		</div>
		<div class="mwb-properties__body mwb-properties__pricebook-form select-pricebook-form">
			
		</div>
		<div class="mwb-properties__body mwb-properties__pricebook-form new-pricebook-form hide">
			<input type="text" value="" name="pricebook_name"
				placeholder="<?php esc_html_e( 'Enter pricebook name', 'integration-with-salesforce' ); ?>"
				id="mwb-woo-salesforce-pricebook-name"/>
		</div>
		<div class="mwb-properties__body mwb-properties__pricebook-form new-pricebook-form hide">
			<a id="mwb-woo-salesforce-create-pricebook-setup" 
				class="mwb-btn mwb-btn--filled create-pricebook">
				<?php esc_html_e( 'Create Pricebook', 'integration-with-salesforce' ); ?>
				</a>
				<a id="mwb-woo-salesforce-back-to-pricebook-select" 
				class="mwb-btn mwb-btn--filled create-pricebook">
				<?php esc_html_e( 'Back', 'integration-with-salesforce' ); ?>
			</a>
		</div>
		<div class="mwb-properties__body mwb-properties__pricebook-success hide">
			<p class="mwb-properties__text">
				<?php
				esc_html_e( 'Pricebook setup completed, please move to next step.', 'integration-with-salesforce' );
				?>
			</p>
			<div class="mwb-properties__button">
				<a href="#" class="mwb-btn mwb-btn--filled progress-next show-properties">
					<?php esc_html_e( 'Move to next step', 'integration-with-salesforce' ); ?>
				</a>
			</div>
		</div>
	</div>
</section>

<section class="mwb-properties">
	<div class="mwb-content-wrap">
		<div class="mwb-properties__header">
			<h2 class="mwb-section__heading">
				<?php esc_html_e( 'Create Feeds', 'integration-with-salesforce' ); ?>
			</h2>
		</div>
		<div class="mwb-properties__body mwb-properties__content">
			<p class="mwb-properties__text">
				<?php
				esc_html_e( 'Next you will need to create some feeds. Feeds are mapping rules, according to which data is being mapped and sent over Salesforce CRM.', 'integration-with-salesforce' );
				?>
			</p>
			<p class="mwb-properties__text">
				<?php
				esc_html_e( 'Integration creates below listed default feeds to sync Products, Accounts, Contacts, Opportunities and Orders over Salesforce CRM.', 'integration-with-salesforce' );
				?>
			</p>
			<ul class="mwb-properties__list">
				<li class="mwb-properties__list-item">
					<?php esc_html_e( 'Account Feed', 'integration-with-salesforce' ); ?>
				</li>
				<li class="mwb-properties__list-item">
					<?php esc_html_e( 'Contact Feed', 'integration-with-salesforce' ); ?>
				</li>
				<li class="mwb-properties__list-item">
					<?php esc_html_e( 'Product Feed', 'integration-with-salesforce' ); ?>
				</li>
				<li class="mwb-properties__list-item">
					<?php esc_html_e( 'PricebookEntry Feed', 'integration-with-salesforce' ); ?>
				</li>
				<li class="mwb-properties__list-item">
					<?php esc_html_e( 'Opportunity Feed', 'integration-with-salesforce' ); ?> 
				</li>
				<li class="mwb-properties__list-item">
					<?php esc_html_e( 'Order Feed', 'integration-with-salesforce' ); ?> 
				</li>
				<li class="mwb-properties__list-item">
					<?php esc_html_e( 'OrderItem Feed', 'integration-with-salesforce' ); ?>
				</li>
			</ul>
			<div class="mwb-properties__progress-button">
				<a href="#" id="mwb-salesforce-create-setup-feed" class="mwb-btn mwb-btn--filled">
					<?php esc_html_e( 'Create Feeds', 'integration-with-salesforce' ); ?>
				</a>
			</div>
			<div class="mwb-properties__progress">
				<div id="mwb-properties__progress-bar" class="mwb-properties__progress-bar" role="progressbar"></div>
			</div>
			<div class="mwb-sync-result">
				<span class="mwb-lists__progress-result">
				</span>
			</div>
			<div class="mwb-properties__button">
				<a href="#" class="mwb-btn mwb-btn--filled progress-next show-sync">
					<?php esc_html_e( 'Move to next step', 'integration-with-salesforce' ); ?>
				</a>
			</div>
		</div>         
	</div>
</section>


<section class="mwb-sync">
	<div class="mwb-content-wrap">
		<div class="mwb-sync__header">
			<h2 class="mwb-section__heading">
				<?php
				esc_html_e( 'Congrats! You’ve successfully set up the Salesforce WooCommerce Integration.', 'integration-with-salesforce' );
				?>
			</h2>
		</div>
		<div class="mwb-sync__body mwb-sync__content-wrap">            
			<div class="mwb-sync__image">    
				<img src="<?php echo esc_url( INTEGRATION_WITH_SALESFORCE_URL . 'assets/src/images/congo.jpg' ); ?>">
			</div>       
			<div class="mwb-sync__content">            
				<p> 
					<?php
					esc_html_e( 'Now you can go to the dashboard and check for the synced data. You can check your feeds, edit them and resync the data if you want. If you do not see your data over Salesforce CRM, you can check the logs for any possible error.', 'integration-with-salesforce' );
					?>
				</p>
				<div class="mwb-sync__button">
					<a href="#" id="mwb-salesforce-onboarding-complete" class="mwb-btn mwb-btn--filled">
						<?php esc_html_e( 'View Dashboard', 'integration-with-salesforce' ); ?>
					</a>
				</div>
			</div>             
		</div>       
	</div>
</section>
