<?php
/**
 * Fired during plugin activation
 *
 * @link       https://wpswings.com/
 * @since      1.0.0
 *
 * @package    Integration_With_Salesforce
 * @subpackage Integration_With_Salesforce/includes
 */

/**
 * Fired during plugin activation.
 *
 * This class defines all code necessary to run during the plugin's activation.
 *
 * @since      1.0.0
 * @package    Integration_With_Salesforce
 * @subpackage Integration_With_Salesforce/includes
 */
class Integration_With_Salesforce_Activator {

	/**
	 * Short Description. (use period)
	 *
	 * Long Description.
	 *
	 * @since    1.0.0
	 */
	public static function activate( $network_wide ) {

		$slug = Integration_With_Salesforce::get_current_crm_name( 'slug' );

		global $wpdb;
		if ( is_multisite() && $network_wide ) {
			if ( ! function_exists( 'is_plugin_active_for_network' ) ) {
				require_once ABSPATH . '/wp-admin/includes/plugin.php';
			}
			$blogids = $wpdb->get_col( "SELECT blog_id FROM $wpdb->blogs" );
			foreach ( $blogids as $blog_id ) {
				switch_to_blog( $blog_id );
				self::may_be_create_log_table( $slug );
				self::add_scheduled_event( $slug );
				restore_current_blog();
			}
			return;
		}
		
		// Create log table in database.
		self::may_be_create_log_table( $slug );

		// Schedule clear log event.
		self::add_scheduled_event( $slug );
	}

	/**
	 * Schedule clear log event.
	 *
	 * @param string $slug Slug of current plugin crm.
	 */
	public static function add_scheduled_event( $slug = '' ) {
		if ( ! wp_next_scheduled( 'mwb_woo_' . $slug . '_clear_log' ) ) {
			wp_schedule_event( time(), 'daily', 'mwb_woo_' . $slug . '_clear_log' );
		}
	}

	/**
	 * Create log table in database.
	 *
	 * @param string $slug Slug of current plugin crm.
	 */
	public static function may_be_create_log_table( $slug = '' ) {

		if ( ! empty( $slug ) ) {
			mwb_woo_salesforce_create_log_table( $slug );
		}
	}
}
