<?php
/**
 * The ajax-specific functionality of the plugin.
 *
 * @link       https://wpswings.com/
 * @since      1.0.0
 *
 * @package    Integration_With_Salesforce
 * @subpackage Integration_With_Salesforce/includes
 */

if ( ! class_exists( 'Integration_With_Salesforce_Framework_Base' ) ) {
	wp_die( 'Integration_With_Salesforce_Framework_Base does not exists.' );
}

/**
 * The ajax-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the ajax-specific stylesheet and JavaScript.
 *
 * @package    Integration_With_Salesforce
 * @subpackage Integration_With_Salesforce/includes
 */
class Integration_With_Salesforce_Ajax_Callbacks {

	/**
	 * Ajax Call back
	 */
	public function mwb_woo_salesforce_ajax() {

		$nonce_key = Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '_ajax_nonce';

		check_ajax_referer( $nonce_key, 'nonce' );

		$event = ! empty( $_POST['event'] ) ? sanitize_text_field( wp_unslash( $_POST['event'] ) ) : '';

		if ( method_exists( $this, $event ) ) {
			$data = $this->$event( $_POST );
		} else {
			$data = esc_html__( 'method not found', 'integration-with-salesforce' );
		}
		echo wp_json_encode( $data );
		wp_die();
	}

	/**
	 * Get objects in crm.
	 *
	 * @param array $posted_data Array of ajax posted data.
	 * @return array $module_data data of specific module.
	 */
	public function get_crm_objects( $posted_data = array() ) {
		$objects        = array();
		$force          = ! empty( $posted_data['force'] ) ? sanitize_text_field( wp_unslash( $posted_data['force'] ) ) : false;
		$response       = array(
			'success' => true,
			'data'    => $objects,
		);
		$crm_api_module = Integration_With_Salesforce::get_integration_module( 'api' );
		$objects        = $crm_api_module->get_crm_objects( $force );
		if ( ! empty( $objects ) ) {
			$response = array(
				'success' => true,
				'data'    => $objects,
			);
		}
		return $response;
	}

	/**
	 * Get fields for a particular object
	 *
	 * @param  array $posted_data Ajax request data.
	 * @return array               Array for fields.
	 */
	public function get_object_fields_for_mapping( $posted_data = array() ) {
		$response    = array( 'success' => false );
		$fields_data = array();
		$object      = ! empty( $posted_data['selected_object'] ) ? sanitize_text_field( wp_unslash( $posted_data['selected_object'] ) ) : '';
		$force       = ! empty( $posted_data['force'] ) ? sanitize_text_field( wp_unslash( $posted_data['force'] ) ) : false;
		$feed_id     = ! empty( $posted_data['post_id'] ) ? sanitize_text_field( wp_unslash( $posted_data['post_id'] ) ) : false;

		$crm_api_module               = Integration_With_Salesforce::get_integration_module( 'api' );
		$fields_data                  = $crm_api_module->get_object_fields( $object, $force );
		$feed_data['crm_fields']      = $fields_data;
		$feed_data['selected_object'] = $object;

		$options = $this->get_field_mapping_options();

		$feed_data['field_options'] = $options;

		return array(
			'html'   => $this->retrieved_html( $feed_id, $feed_data ),
			'fields' => $fields_data,
		);
	}

	/**
	 * Get all mapping options for a field.
	 *
	 * @return array Array for field option.
	 */
	public function get_field_mapping_options() {
		$framework_instance = Integration_With_Salesforce::get_integration_module( 'framework' );
		$options            = $framework_instance->getMappingDataset();
		return $options;
	}

	/**
	 * Ajax Callbacks for gettings module HTML.
	 *
	 * @param  int   $feed_id Feed id.
	 * @param  array $posted_data Posted data.
	 * @return string             Response html.
	 */
	public function retrieved_html( $feed_id, $posted_data ) {

		$feed_module     = Integration_With_Salesforce::get_integration_module( 'feed' );
		$event_options   = $feed_module->get_feed_event_options( array(), $feed_id );
		$selected_object = $posted_data['selected_object'];
		$feed_event      = $feed_module->get_feed_data( $feed_id, 'feed_event', '', $selected_object );
		$primary_field   = $feed_module->get_feed_data( $feed_id, 'primary_field', '', $selected_object );
		$mapping_data    = $feed_module->get_feed_data( $feed_id, 'mapping_data', '', $selected_object );

		$params = array(
			'selected_object'         => $selected_object,
			'crm_fields'              => $posted_data['crm_fields'],
			'field_options'           => $posted_data['field_options'],
			'mapping_data'            => $mapping_data,
			'primary_field'           => $primary_field,
			'event_options'           => $event_options,
			'feed_event'              => $feed_event,
			'status_mapping'          => false,
			'status_mapping_field'    => false,
			'pricebook_mapping_field' => false,
			'account_person_field'    => false,
		);

		$templates = array(
			'select-fields',
			'add-new-field',
			'primary-field',
			'select-event',
			'nonce-field',
		);

		$status_mapping_field    = $feed_module->if_object_support_status_mapping( $selected_object );
		$pricebook_mapping_field = $feed_module->if_object_support_pricebook_mapping( $selected_object );

		if ( ! empty( $status_mapping_field ) ) {
			$order_statuses                 = $feed_module->get_woo_order_statuses();
			$params['order_statuses']       = $order_statuses;
			$params['status_mapping_field'] = $status_mapping_field;
			if ( ! $feed_id ) {
				$params['status_mapping'] = $feed_module->get_default_status_mapping( $selected_object, $status_mapping_field, $order_statuses );
			} else {
				$params['status_mapping'] = $feed_module->get_feed_data( $feed_id, 'status_mapping', '' );
			}
			$templates[] = 'order-status-mapping';
		}

		if ( 'Product2' === $selected_object ) {
			$templates[]                   = 'select-pricebook';
			$templates[]                   = 'sync-variable-product';
			$params['sync_parent_product'] = $feed_module->get_feed_data( $feed_id, 'sync_parent_product', 'no' );
		}

		if ( 'Product2' === $selected_object || ! empty( $pricebook_mapping_field ) ) {
			$params['pricebook_options']       = $this->get_pricebook_options()['data'];
			$params['selected_pricebook']      = $feed_module->get_feed_data( $feed_id, 'selected_pricebook', '' );
			$params['pricebook_mapping_field'] = $pricebook_mapping_field;
		}

		if ( 'Account' === $selected_object ) {
			$templates[]                   = 'select-person-account';
			$params['sync_person_account'] = $feed_module->get_feed_data( $feed_id, 'sync_person_account', 'no' );
		}

		$html = '';
		foreach ( $templates as $k => $v ) {
			$html .= $feed_module->do_ajax_render( $v, $params );
		}
		return $html;
	}

	/**
	 * Get pricebook option from Salesforce.
	 *
	 * @param  array $posted_data Posted data.
	 * @return Array Pricebook Options Array.
	 */
	public function get_pricebook_options( $posted_data = array() ) {
		$pricebook_data = array();
		$response       = array(
			'success' => false,
			'data'    => $pricebook_data,
		);
		$force          = isset( $posted_data['force'] ) ? $posted_data['force'] : false;
		$crm_api_module = Integration_With_Salesforce::get_integration_module( 'api' );
		$pricebook_data = $crm_api_module->get_pricebook_data( $force );
		update_option( 'mwb_salesforce_pricebook_data', $pricebook_data );
		if ( ! empty( $pricebook_data ) ) {
			$response = array(
				'success' => true,
				'data'    => $pricebook_data,
			);
		}
		return $response;
	}

	/**
	 * Add new field in feed form.
	 *
	 * @param  array $posted_data Posted data.
	 * @return array              Response data.
	 */
	public function add_new_field( $posted_data ) {
		$response       = array( 'success' => false );
		$object         = $posted_data['object'];
		$field          = $posted_data['field'];
		$crm_api_module = Integration_With_Salesforce::get_integration_module( 'api' );
		$fields_data    = $crm_api_module->get_object_fields( $object, false );
		$field_index    = array_search( $field, array_column( $fields_data, 'name' ) ); // phpcs:ignore
		if ( false === $field_index ) {
			return $response;
		}

		$framework_instance = Integration_With_Salesforce::get_integration_module( 'framework' );
		$field_options      = $framework_instance->getMappingDataset();

		$require_status_mapping    = false;
		$require_pricebook_mapping = false;

		$feed_module          = Integration_With_Salesforce::get_integration_module( 'feed' );
		$status_mapping_field = $feed_module->if_object_support_status_mapping( $object );
		if ( $status_mapping_field === $fields_data[ $field_index ]['name'] ) {
			$require_status_mapping = true;
		}
		$pricebook_mapping_field = $feed_module->if_object_support_pricebook_mapping( $object );
		$pricebook_options       = $this->get_pricebook_options()['data'];
		if ( ! empty( $pricebook_options ) && $pricebook_mapping_field === $fields_data[ $field_index ]['name'] ) {
			$require_pricebook_mapping = true;
			$field_options             = $pricebook_options;
		}
		ob_start();
		Integration_With_Salesforce_Framework::get_field_section_html( $field_options, $fields_data[ $field_index ], array(), $require_status_mapping, $require_pricebook_mapping, $object );
		$output = ob_get_contents();
		ob_end_clean();
		$response = array(
			'success' => true,
			'html'    => $output,
		);
		return $response;
	}

	/**
	 * Add new field in feed form.
	 *
	 * @param  array $posted_data Posted data.
	 * @return array              Response data.
	 */
	public function create_new_pricebook( $posted_data = array() ) {
		$response = array(
			'success' => false,
			'msg'     => __( 'Something went wrong', 'integration-with-salesforce' ),
		);

		$pricebook_name = isset( $posted_data['name'] ) ? $posted_data['name'] : '';
		if ( '' == $pricebook_name ) {
			return $response;
		}

		$params = array( 'Name' => $pricebook_name );

		$crm_api_module = Integration_With_Salesforce::get_integration_module( 'api' );
		$response_data  = $crm_api_module->create_or_update_record( 'Pricebook2', $params );

		if ( $response_data['success'] ) {
			$response = $response_data;
		}
		return $response;
	}

	/**
	 * Delete feed post.
	 *
	 * @param  array $posted_data Post data.
	 * @return array             Response data.
	 */
	public function delete_feed_post( $posted_data = array() ) {

		$feed_id = isset( $posted_data['feed_id'] ) ? sanitize_text_field( wp_unslash( $posted_data['feed_id'] ) ) : false;
		if ( $feed_id ) {
			wp_trash_post( $feed_id );
		}
		return array( 'success' => true );
	}


	/**
	 * Change feed status.
	 *
	 * @param  array $posted_data Post data.
	 * @return array             Response data.
	 */
	public function change_feed_status( $posted_data = array() ) {

		$feed_id = isset( $posted_data['feed_id'] ) ? sanitize_text_field( wp_unslash( $posted_data['feed_id'] ) ) : false;
		$status  = isset( $posted_data['status'] ) ? sanitize_text_field( wp_unslash( $posted_data['status'] ) ) : false;
		if ( $feed_id && $status ) {
			$args = array(
				'ID'          => $feed_id,
				'post_status' => $status,
			);
			wp_update_post( $args );
		}
		return array( 'success' => true );
	}

	/**
	 * Disconnect salesforce account
	 *
	 * @return array Response data.
	 */
	public function disconnect_account() {

		$wipe_allowed = Integration_With_Salesforce_Admin::is_wipe_allowed();
		$crm          = Integration_With_Salesforce::get_current_crm_name( 'slug' );
		$result       = mwb_woo_salesforce_get_all_options( $crm );

		if ( true === $wipe_allowed ) {
			$saved_post_meta = mwb_woo_crm_saved_feed_meta();
			if ( ! empty( $saved_post_meta ) && is_array( $saved_post_meta ) ) {
				foreach ( $saved_post_meta as $key => $meta ) {
					delete_post_meta( $meta['post_id'], $meta['meta_key'] );
				}
			}
		}

		if ( ! empty( $result ) && is_array( $result ) ) {

			foreach ( $result as $key => $option ) {
				if ( ! empty( $option['option_name'] ) ) {
					delete_option( $option['option_name'] );
				}
			}
		}

		return array( 'success' => true );
	}

	/**
	 * Get api authorization url
	 *
	 * @return array Response data.
	 */
	public function get_reauthorize_url() {
		$response = array( 'success' => false );

		$crm_api_module = Integration_With_Salesforce::get_integration_module( 'api' );
		$auth_url       = $crm_api_module->get_auth_code_url();

		if ( ! $auth_url ) {
			$response['msg'] = __( 'Something went wrong! Check your credentials and authorize again', 'integration-with-salesforce' );
			delete_option( 'mwb-woo-salesforce-authorised', false );
			return $response;
		}
		$response = array(
			'success' => true,
			'url'     => $auth_url,
		);
		return $response;
	}

	/**
	 * Get log table data
	 */
	public function get_datatable_data_cb() {
		check_ajax_referer( 'ajax_nonce', 'nonce' );
		$request = $_GET;
		$offset  = $request['start'];
		$limit   = $request['length'];

		$search_data  = $request['search'];
		$search_value = $search_data['value'];

		// Custom error filter.
		$search_by_error = $request['searchByError'];

		$total_recordwith_filter = 0;

		$search_query = ' ';
		$count_data   = mwb_woo_salesforce_get_total_log_count( $search_query );
		$total_count  = $count_data[0];
		$data         = array();

		if ('error' !== $search_by_error && '' !== $search_by_error ) {

			$search_query =  $search_by_error;
			$log_data = mwb_woo_salesforce_get_feed_log_data( $search_by_error, $limit, $offset);
			// Total number of records with filtering.
			$total_recordwith_filter = mwb_woo_salesforce_get_total_log_count_feed_filters( $search_query );
		} elseif ( 'error' === $search_by_error ) {
			$search_query .= " and (salesforce_id = '-' or salesforce_id = '') ";
			$log_data      = mwb_woo_salesforce_get_error_data( 'error', $limit, $offset );
			// Total number of records with filtering.
			$total_recordwith_filter = mwb_woo_salesforce_get_total_log_count( $search_query );
		} elseif ( '' !== $search_value ) {
			$search_query .= " and (woo_id = $search_value) ";
			$log_data      = mwb_woo_salesforce_get_log_data( $search_value, $limit, $offset );
			// Total number of records with filtering.
			$total_recordwith_filter = mwb_woo_salesforce_get_total_log_count( $search_query );
		} else {

			$log_data = mwb_woo_salesforce_get_log_data( false, $limit, $offset );
			// Total number of records with filtering.
			$total_recordwith_filter = mwb_woo_salesforce_get_total_log_count( $search_query );
		}

		foreach ( $log_data as $key => $value ) {

			$value['salesforce_id'] = ! empty( $value['salesforce_id'] ) ? $value['salesforce_id'] : '-';

			$temp   = array(
				'',
				$value['event'],
				$value['woo_object'],
				$value['woo_id'],
				$value['salesforce_object'],
				$value['salesforce_id'],
				gmdate( 'd-m-Y h:i A', esc_html( $value['time'] ) ),
				$value['request'],
				$value['response'],
			);
			$data[] = $temp;
		}

		$json_data = array(
			'draw'            => intval( $request['draw'] ),
			'recordsTotal'    => $total_count,
			'recordsFiltered' => $total_recordwith_filter,
			'data'            => $data,
		);

		echo wp_json_encode( $json_data );

		wp_die();
	}

	/**
	 * Renew access token.
	 *
	 * @return array Response data.
	 */
	public function renew_access_token() {
		$response       = array( 'success' => false );
		$crm_api_module = Integration_With_Salesforce::get_integration_module( 'api' );
		$renew_result   = $crm_api_module->renew_access_token();

		$response['msg'] = __( 'Something went wrong! Check your credentials and authorize again', 'integration-with-salesforce' );
		if ( ! empty( $renew_result ) && true == $renew_result ) {
			$issue_time    = $crm_api_module->get_access_token_issue_time();
			$access_token  = $crm_api_module->get_access_token();
			$token_message = sprintf( 'Last token issue time : %s.', gmdate( 'd-m-Y h:i A', $issue_time ) );
			$response      = array(
				'success'       => true,
				'msg'           => __( 'Success', 'integration-with-salesforce' ),
				'token_message' => $token_message,
				'access_token'  => $access_token,
			);
		}
		return $response;
	}

	/**
	 * Get default feeds data.
	 *
	 * @param  array $posted_data Post data.
	 * @return array             Response data.
	 */
	public function get_default_feeds_data( $posted_data ) {

		$response = array(
			'success' => false,
			'data'    => array(),
		);

		$feed_module = Integration_With_Salesforce::get_integration_module( 'feed' );
		$feeds       = $feed_module->get_default_feeds();

		if ( ! empty( $feeds ) ) {
			$response['success'] = true;
			$response['data']    = $feeds;
		}

		return $response;
	}

	/**
	 * Create default feeds.
	 *
	 * @param  array $posted_data Post data.
	 * @return array              Response data.
	 */
	public function create_default_feed( $posted_data = array() ) {

		$response = array(
			'success' => false,
			'msg'     => __( 'Something went wrong !', 'integration-with-salesforce' ),
		);

		$feed_module = Integration_With_Salesforce::get_integration_module( 'feed' );
		$feeds       = $feed_module->get_default_feeds();
		$step        = isset( $posted_data['step'] ) ? $posted_data['step'] : -1;
		$feed        = ! empty( $feeds[ $step ] ) ? $feeds[ $step ] : array();

		if ( empty( $feed ) ) {
			return $response;
		}

		$existing_feed = get_option( 'mwb_woo_salesforce_' . $feed['object'] . '_feed_id', false );

		$feed_id = false;

		// Check if found existing feed.
		if ( false !== $existing_feed ) {
			// Found existing feed valid it.
			$feed_id = $feed_module->is_valid_feed_post( $existing_feed );
		}
		if ( false === $feed_id ) {
			// Check for feed post in all post.
			// Extra check if feed id is deleted from options.
			$feed_id = $feed_module->check_for_feed_post( $feed['object'] );
		}
		if ( false === $feed_id ) {
			// Still not found the feed, create it now.
			$feed_id = $feed_module->create_feed_post( $feed['title'], $feed['object'] );

			if ( ! is_wp_error( $feed_id ) && false !== $feed_id ) {
				$response['msg'] = sprintf(
					/* translators: Feed object name */
					__( '%s feed created.', 'integration-with-salesforce' ),
					$feed['object']
				);
				$response['success'] = true;
			}
		} else {
			$response['msg'] = sprintf(
				/* translators: Feed object name */
				__( '%s feed already exists.', 'integration-with-salesforce' ),
				$feed['object']
			);
			$response['success'] = true;
		}

		update_option( 'mwb_woo_salesforce_' . $feed['object'] . '_feed_id', $feed_id );

		if ( ( $step + 1 ) === count( $feeds ) ) {
			update_option( 'mwb_woo_salesforce_feeds_created', true );
		}

		return $response;
	}

	/**
	 * Update option to mark onboarding complete.
	 *
	 * @param array $posted_data Array of ajax posted data.
	 * @return array Response array.
	 */
	public function mark_onboarding_complete( $posted_data = array() ) {
		update_option( 'mwb-woo-salesforce-onboarding-completed', true );
		return array( 'success' => true );
	}

	/**
	 * Save plugin general settings
	 *
	 * @param array $posted_data Array of ajax posted data.
	 * @return array Response array.
	 */
	public function save_general_setting( $posted_data = array() ) {

		$admin_instance = new Integration_With_Salesforce_Admin( 'Salesforce Integration for WooCommerce', '1.2.1' );
		$settings = $posted_data['settings'];
		update_option( 'mwb-salesforce-enable-sync', sanitize_text_field( wp_unslash( $settings['sync_enable'] ) ) );
		update_option( 'mwb-salesforce-background-sync-allowed', sanitize_text_field( wp_unslash( $settings['background_sync'] ) ) );
		update_option( 'mwb-salesforce-enable-log', sanitize_text_field( wp_unslash( $settings['log_enable'] ) ) );
		update_option( 'mwb-salesforce-delete-log', sanitize_text_field( wp_unslash( $settings['log_delete'] ) ) );
		update_option( 'mwb-salesforce-wipe-data', sanitize_text_field( wp_unslash( $settings['wipe_data'] ) ) );

		if ( $admin_instance->wps_is_woocommerce_subscription_plugin_active() ) {
			update_option( 'mwb-salesforce-sync-renewal', sanitize_text_field( wp_unslash( $settings['sync_renewal'] ) ) );
		}
		update_option( 'mwb-salesforce-update-product-stock', sanitize_text_field( wp_unslash( $settings['product_stock'] ) ) );
		update_option( 'mwb-salesforce-update-product-stock-api-name', sanitize_text_field( wp_unslash( $settings['product_stock_field_name'] ) ) );
		update_option( 'mwb-salesforce-enable-product-deletion-woo-to-salesforce', sanitize_text_field( wp_unslash( $settings['prod_deletion_woo_to_salesforce'] ) ) );
		update_option( 'mwb-salesforce-enable-product-deletion-salesforce-to-woo', sanitize_text_field( wp_unslash( $settings['prod_deletion_salesforce_to_woo'] ) ) );
		return array(
			'success' => true,
			'message' => __( 'Settings Saved', 'integration-with-salesforce' ),
		);
	}

	/**
	 * Clear sync log.
	 *
	 * @return array Response array.
	 */
	public function clear_sync_log() {
		Integration_With_Salesforce_Admin::delete_sync_log();
		return array( 'success' => true );
	}

	/**
	 * Download sync log.
	 *
	 * @return array Response array.
	 */
	public function download_sync_log() {

		$crm_slug = Integration_With_Salesforce::get_current_crm_name( 'slug' );
		$crm_name = Integration_With_Salesforce::get_current_crm_name();
		$log_data = mwb_woo_salesforce_get_log_data( false, 25, 0, true );
		$data     = array();
		$log_dir  = WC_LOG_DIR . 'mwb-woo-' . $crm_slug . '-sync-log.log';
		if ( ! is_dir( $log_dir ) ) {
			$log_dir = WC_LOG_DIR . 'mwb-woo-' . $crm_slug . '-sync-log.log';
		}

		global $wp_filesystem;  // Define global object of WordPress filesystem.
		WP_Filesystem();        // Intialise new file system object.

		$file_data = '';
		$wp_filesystem->put_contents( $log_dir, '' );

		foreach ( $log_data as $key => $value ) {
			$log  = 'FEED : ' . $value['event'] . PHP_EOL;
			$log .= 'WOO ID : ' . $value['woo_id'] . PHP_EOL;
			$log .= 'WOO OBJECT : ' . $value['woo_object'] . PHP_EOL;
			$log .= $crm_name . ' ID : ' . $value[ $crm_slug . '_id' ] . PHP_EOL;
			$log .= $crm_name . ' OBJECT : ' . $value[ $crm_slug . '_object' ] . PHP_EOL;
			$log .= 'TIME : ' . gmdate( 'd-m-Y h:i A', esc_html( $value['time'] ) ) . PHP_EOL;
			$log .= 'REQUEST : ' . $value['request'] . PHP_EOL;
			$log .= 'RESPONSE : ' . $value['response'] . PHP_EOL;
			$log .= '------------------------------------' . PHP_EOL;

			$file_data .= $log;

		}
		$wp_filesystem->put_contents( $log_dir, $file_data );
		return array(
			'success'  => true,
			'redirect' => admin_url( '?mwb_download=sync' ),
		);
	}

	/**
	 * Sync particular object manually.
	 *
	 * @param array $posted_data Array of ajax posted data.
	 *
	 * @since 1.0.0
	 *
	 * @return bool|array feed ids with label.
	 */
	public function sync_object_manually( $posted_data = array() ) {

		$feed_id = ! empty( $posted_data['feed_id'] ) ?
		$posted_data['feed_id'] :
		false;

		$post_id = ! empty( $posted_data['post_id'] ) ?
		$posted_data['post_id'] :
		false;

		if ( empty( $feed_id ) || empty( $post_id ) ) {
			return array(
				'status'  => 404,
				'message' => esc_html__( 'Required data not found.', 'integration-with-salesforce' ),
			);
		} else {

			$_post          = get_post( $post_id );
			$post_type      = ! empty( $_post ) ? $_post->post_type : false;
			$salesforce     = Integration_With_Salesforce::get_integration_module( 'framework' );
			$request_module = Integration_With_Salesforce::get_integration_module( 'request' );

			$record_type = $salesforce->get_feed( $feed_id, 'crm_object' );
			$feed_title  = $salesforce->get_feed_title( $feed_id );

			switch ( $record_type ) {
				case 'Order':
				case 'Opportunity':
				case 'Contact':
				case 'Account':
				case 'Case':
				case 'Quote':
				case 'Lead':
					$request_module->perform_shop_order_sync( $feed_id, $post_id );
					break;

				case 'Product2':
				case 'PricebookEntry':
					$request_module->trigger_product_related_feed( $feed_id, $post_id, 'true' );
					break;
				case 'OrderItem':
				case 'OpportunityLineItem':
				case 'QuoteLineItem':
					$request_module->trigger_order_items_feed( $feed_id, $post_id );
					break;
			}

			return array(
				'status'  => 200,
				'message' => esc_html__( 'Sync Completed. Check Logs for verification.', 'integration-with-salesforce' ),
			);
		}
	}

	/**
	 * Get all object id's required for sync.
	 *
	 * @param array $posted_data Array of post data.
	 * @return array             Response data.
	 */
	public function fetch_required_sync_count( $posted_data = array() ) {
		$response = array(
			'success' => false,
			'message' => __( 'Something went wrong !!', 'integration-with-salesforce' ),
		);

		$feed_id   = isset( $posted_data['feed_id'] ) ? $posted_data['feed_id'] : false;
		$sync_type = isset( $posted_data['sync_type'] ) ? $posted_data['sync_type'] : 'bulk_sync';
		if ( ! $feed_id ) {
			return $response;
		}

		$request_module = Integration_With_Salesforce::get_integration_module( 'request' );
		$woo_object_ids = $request_module->get_feed_required_object_ids( $feed_id, $sync_type );

		if ( ! empty( $woo_object_ids ) ) {
			$response['data']    = $woo_object_ids;
			$response['success'] = true;
			return $response;
		}
		$response['message'] = __( 'No data found. Seems everything is up to date', 'integration-with-salesforce' );
		return $response;
	}

	/**
	 * Get all object id's required for sync.
	 *
	 * @param array $posted_data Array of post data.
	 * @return array             Response data.
	 */
	public function sync_bulk_data_request( $posted_data = array() ) {

		$response = array(
			'success'       => false,
			'data'          => array(),
			'has_next_step' => false,
		);

		$feed_id   = isset( $posted_data['feed_id'] ) ? $posted_data['feed_id'] : false;
		$step      = isset( $posted_data['step'] ) ? $posted_data['step'] : false;
		$sync_type = isset( $posted_data['sync_type'] ) ? $posted_data['sync_type'] : 'bulk_sync';
		if ( false === $feed_id || false === $step ) {
			return $response;
		}
		$length           = 10;
		$start            = $step * $length;
		$salesforce       = Integration_With_Salesforce::get_integration_module( 'framework' );
		$record_type      = $salesforce->get_feed( $feed_id, 'crm_object' );
		$option_key       = 'mwb_woo_salesforce_' . $record_type . '_' . $sync_type . '_sync_ids';
		$saved_object_ids = get_option( $option_key, array() );
		
		$current_dataset = array_slice( $saved_object_ids, $start, $length );
		if ( ! empty( $current_dataset ) ) {
			$sync_data = $this->temp_bulk_data_sync( $feed_id, $current_dataset );
		

			$percent  = ceil( ( $start + count( $current_dataset ) ) / count( $saved_object_ids ) * 100 );
			$response = array(
				'success'       => true,
				'data'          => $current_dataset,
				'sync_data'     => $sync_data,
				'next_step'     => ++$step,
				'has_next_step' => true,
				'percent'       => $percent,
			);
		}
		
		return $response;
	}

	/**
	 * Bulk data sync.
	 *
	 * @param int   $feed_id         Feed id.
	 * @param array $current_dataset Dataset to be synced.
	 * @return array                 Response data.
	 */
	public function temp_bulk_data_sync( $feed_id, $current_dataset ) {

		$feed_module        = Integration_With_Salesforce::get_integration_module( 'feed' );
		$framework_instance = Integration_With_Salesforce::get_integration_module( 'framework' );
		$crm_api_module     = Integration_With_Salesforce::get_integration_module( 'api' );
		$request_module     = Integration_With_Salesforce::get_integration_module( 'request' );

		
		$bulk_data = $request_module->get_bulk_data( $feed_id, $current_dataset );
		$request_data = array();
		$result_array = array();
		$dependencies = $feed_module->get_feed_dependencies( $bulk_data['object'] );
		$buld_ids     = array();
		$filter_exist = $request_module->maybe_check_filter( $feed_id );
		// Logic to prevent multiple Account for same order email.
		// @todo  Create a new function for this.

		$visited          = array();
		$relationship     = array();
		$new_request_data = $bulk_data['request'];
		$object_type = $request_module->get_woo_object_based_on_event($feed_id);
		if ( ( 'Account' === $bulk_data['object'] || 'Contact' === $bulk_data['object'] || 'Lead' === $bulk_data['object'] )&& ( 'shop_order' == $bulk_data['woo_object'] ) ) {
			$visited          = array();
			$relationship     = array();
			$new_request_data = array();
			foreach ( $bulk_data['request'] as $k1 => $data1 ) {
				if ( ! in_array( $data1['post_id'], $visited ) ) {
					$visited[]                         = $data1['post_id'];
					$new_request_data[]                = $data1;
					$relationship[ $data1['post_id'] ] = array();
					if ( ! empty( $data1['related_order_ids'] ) ) {
						foreach ( $data1['related_order_ids'] as $rk => $rd ) {
							if ( ! in_array( $rd, $visited ) && in_array( $rd, $bulk_data['bulk_ids'] ) ) {
								$visited[]                           = $rd;
								$relationship[ $data1['post_id'] ][] = $rd;
							}
						}
					}
				}
			}
		}

		// Need to put this code block in a function.

		if ( 'OrderItem' === $bulk_data['object'] || 'OpportunityLineItem' === $bulk_data['object'] || 'QuoteLineItem' === $bulk_data['object'] ) {
			foreach ( $new_request_data as $k => $data ) {
				$relationship[ $data['item_id'] ] = $data['post_id'];
				if ( ! empty( $dependencies ) ) {
					$flag = true;
					foreach ( $dependencies as $dk => $dv ) {
						if ( empty( $data[ $dk ] ) ) {
							$result_array[ $data['item_id'] ] = array(
								'id'      => '',
								'post_id' => $data['post_id'],
								'message' => 'Missing Dependency : ' . $dk,
							);
							$flag                             = false;
						}
					}
					if ( $flag ) {
						if ( ! empty( $data['id'] ) ) {
							$result_array[ $data['item_id'] ] = array(
								'id'      => '',
								'post_id' => $data['post_id'],
								'message' => 'Items aleady synced',
							);
						} else {
							$buld_ids[] = $data['item_id'];
							unset( $data['item_id'] );
							unset( $data['post_id'] );
							$request_data[] = $data;
						}
					}
				} elseif ( ! empty( $data['id'] ) ) {
						$result_array[ $data['item_id'] ] = array(
							'id'      => '',
							'post_id' => $data['post_id'],
							'message' => 'Items aleady synced',
						);
				} else {
					$buld_ids[] = $data['item_id'];
					unset( $data['item_id'] );
					unset( $data['post_id'] );
					$request_data[] = $data;
				}
			}
		} else {
			foreach ( $new_request_data as $k => $data ) {
				if ( ! empty( $dependencies ) ) {
					$flag = true;
					foreach ( $dependencies as $dk => $dv ) {
						if ( empty( $data[ $dk ] ) ) {
							if ('users' == $bulk_data['woo_object'] ) {
								$result_array[ $data['user_id'] ] = array(
									'id'      => '',
									'post_id' => $data['user_id'],
									'message' => 'Missing Dependency : ' . $dk,
								);
							} else {

								$result_array[ $data['post_id'] ] = array(
									'id'      => '',
									'post_id' => $data['post_id'],
									'message' => 'Missing Dependency : ' . $dk,
								);
							}
							$flag                             = false;
						}
					}
					if ( $flag ) {
						if ('users' == $bulk_data['woo_object'] ) {

							$buld_ids[] = $data['user_id'];
						} else {
							$buld_ids[] = $data['post_id'];
						}
						if ( 'OrderItem' === $bulk_data['object'] || 'OpportunityLineItem' === $bulk_data['object'] || 'QuoteLineItem' === $bulk_data['object'] ) {
							unset( $data['item_id'] );
							if ( ! empty( $data['id'] ) ) {
								$result_array[ $data['post_id'] ] = array(
									'id'      => '',
									'post_id' => $data['post_id'],
									'message' => 'Items aleady synced',
								);
							} else {
								unset( $data['post_id'] );
								if ( isset( $data['related_order_ids'] ) ) {
									unset( $data['related_order_ids'] ); // Account.
								}
								$request_data[] = $data;
							}
						} else {
							if ('users' == $bulk_data['woo_object'] ) {
								unset( $data['user_id'] );
							} else {
								unset( $data['post_id'] );
								if ( isset( $data['related_order_ids'] ) ) {
									unset( $data['related_order_ids'] ); // Account.
								}
							}
							$request_data[] = $data;
						}
					}
				} else {
					if ('users' == $bulk_data['woo_object'] ) {

						$buld_ids[] = $data['user_id'];
					} else {
						$buld_ids[] = $data['post_id'];
					}
					if ( 'OrderItem' === $bulk_data['object'] || 'OpportunityLineItem' === $bulk_data['object'] || 'QuoteLineItem' === $bulk_data['object'] ) {
						unset( $data['item_id'] );
						if ( ! empty( $data['id'] ) ) {
							$result_array[ $data['post_id'] ] = array(
								'id'      => '',
								'message' => 'Items aleady synced',
							);
						} else {
							unset( $data['post_id'] );
							if ( isset( $data['related_order_ids'] ) ) {
								unset( $data['related_order_ids'] ); // Account.
							}
							$request_data[] = $data;
						}
					} else {
						if ('users' == $bulk_data['woo_object'] ) {
							unset( $data['user_id'] );
						} else {
							unset( $data['post_id'] );
							if ( isset( $data['related_order_ids'] ) ) {
								unset( $data['related_order_ids'] ); // Account.
							}
						}
						$request_data[] = $data;
					}
				}
			}
		}

		$result = array();

		if ( ! empty( $request_data ) && is_array( $request_data ) ) {
			$result = $crm_api_module->bulk_create_or_update_record( $bulk_data['object'], $request_data, $bulk_data['primary_field'], $feed_id );
			if ( ! empty( $filter_exist ) ) {
				$filtered_ids = array();
				if ( ! empty( $new_request_data ) ) {
					foreach ( $new_request_data as $key => $data ) {
						if ('users'== $bulk_data['woo_object'] ) {
							$object_id = $data['user_id'];
						} elseif ( 'OrderItem' === $bulk_data['object'] || 'OpportunityLineItem' === $bulk_data['object'] || 'QuoteLineItem' === $bulk_data['object'] ) {
							$object_id = $data['item_id'];
						} else {
							$object_id = $data['post_id'];
						}
						$filter_result = $request_module->mwb_salesforce_validate_filter( $filter_exist, $data );
						if ( true === $filter_result ) {
							array_push( $filtered_ids, $object_id );

						} elseif ( is_array( $filter_result ) && false == $filter_result['result'] ) { // phpcs:ignore
							array_push( $filtered_ids, $object_id );
						}
					}
				}
				$buld_ids = $filtered_ids;
			} else {
				$buld_ids = $buld_ids;
			}
			if ( ! empty( $result ) ) {
				if ( count( $buld_ids ) == count( $result ) ) {
					$result = array_combine( $buld_ids, $result );
				}
			}
		}
		$result_array = array_replace( $result_array, $result );

		if ( 'PricebookEntry' == $bulk_data['object'] ) {
			
			$standard_pricebook_error = false;
			foreach ( $result_array as $key => $line_result ) {
				if ( isset( $line_result['success'] ) && ! $line_result['success'] ) {
					if ( ! empty( $line_result['errors'] ) && ! empty( $line_result['errors'][0] ) ) {
						if ( 'STANDARD_PRICE_NOT_DEFINED' === $line_result['errors'][0]['statusCode'] ) {
							$standard_pricebook_error = true;
							
							break;
						}
					}
				}
			}
			
			if ( $standard_pricebook_error ) {
				
				$new_request_data = array();
				// $standard_pricebook_id = '01s5g00000D6GvqAAF'; // Save this while creating pricebook.
				$standard_pricebook_id = get_option( 'mwb_salesforce_standard_pricebook', '' );
				foreach ( $request_data as $key => $request_item ) {
					$request_item['Pricebook2Id'] = $standard_pricebook_id;
					array_push( $new_request_data, $request_item );
				}

				if ( ! empty( $new_request_data ) && is_array( $new_request_data ) ) {
					$new_result = $crm_api_module->bulk_create_or_update_record( $bulk_data['object'], $new_request_data, $bulk_data['primary_field'], $feed_id );
					$result     = $crm_api_module->bulk_create_or_update_record( $bulk_data['object'], $request_data, $bulk_data['primary_field'], $feed_id );
					if ( ! empty( $filter_exist ) ) {
						$filtered_ids = array();
						if ( ! empty( $new_request_data ) ) {
							foreach ( $new_request_data as $key => $data ) {
								if ( 'OrderItem' === $bulk_data['object'] || 'OpportunityLineItem' === $bulk_data['object'] || 'QuoteLineItem' === $bulk_data['object'] ) {
									$object_id = $data['item_id'];
								} else {
									$object_id = $data['post_id'];
								}
								$filter_result = $request_module->mwb_salesforce_validate_filter( $filter_exist, $data );
								if ( true === $filter_result ) {
									array_push( $filtered_ids, $object_id );

								} elseif ( is_array( $filter_result ) && false == $filter_result['result'] ) { // phpcs:ignore
									array_push( $filtered_ids, $object_id );
								}
							}
						}
						$buld_ids = $filtered_ids;
					} else {
						$buld_ids = $buld_ids;
					}
					if ( ! empty( $result ) ) {
						if ( count( $buld_ids ) == count( $result ) ) {
							$result = array_combine( $buld_ids, $result );
						}
					}
				}
				$result_array = array_replace( $result_array, $result );
			}
		}

		$log_data = $result_array;

		$order_item_sync_meta = array();
		foreach ( $result_array as $post_id => $line_response ) {
			if ( ! empty( $line_response['id'] ) ) {
				if ( 'users' == $bulk_data['woo_object'] ) {
					$log_data[ $post_id ] = array_merge( $line_response, array( 'user_id' => $post_id ) );
					update_user_meta( $post_id, 'mwb_salesforce_' . $bulk_data['object'] . '_id', $line_response['id'] );
				} elseif ( 'OrderItem' === $bulk_data['object'] || 'OpportunityLineItem' === $bulk_data['object'] || 'QuoteLineItem' === $bulk_data['object'] ) {
					$order_item_sync_meta[ $relationship[ $post_id ] ][] = $line_response;
					$log_data[ $post_id ]                                = array_merge( $line_response, array( 'post_id' => $relationship[ $post_id ] ) );
				} else {
					$log_data[ $post_id ] = array_merge( $line_response, array( 'post_id' => $post_id ) );
					if (Integration_With_Salesforce_Order_Handler::wps_get_cot_enabled_order_enabled() ) {
						if ( 'Product2'== $bulk_data['object'] || 'PricebookEntry' == $bulk_data['object']) {
							update_post_meta( $post_id, 'mwb_salesforce_' . $bulk_data['object'] . '_id', $line_response['id'] );
						} else {
							$order = wc_get_order( $post_id );
							if ( $order ) {

								$order->update_meta_data( 'mwb_salesforce_' . $bulk_data['object'] . '_id', $line_response['id'] );
								$order->save();
							} else {
								update_post_meta( $post_id, 'mwb_salesforce_' . $bulk_data['object'] . '_id', $line_response['id'] );
							}
						}
					} else {
						update_post_meta( $post_id, 'mwb_salesforce_' . $bulk_data['object'] . '_id', $line_response['id'] );
					}
					if ( 'Account' === $bulk_data['object'] || 'Contact' === $bulk_data['object'] || 'Lead' === $bulk_data['object'] ) {
						if ( ! empty( $relationship[ $post_id ] ) ) {
							foreach ( $relationship[ $post_id ] as $k3 => $v3 ) {
								$log_data[ $v3 ] = array_merge( $line_response, array( 'post_id' => $v3 ) );
								if ( Integration_With_Salesforce_Order_Handler::wps_get_cot_enabled_order_enabled() ) {
									$order = wc_get_order( $v3 );
									if ( $order ) {

										$order->update_meta_data( 'mwb_salesforce_' . $bulk_data['object'] . '_id', $line_response['id'] );
										$order->save();
									} else {
										update_post_meta( $v3, 'mwb_salesforce_' . $bulk_data['object'] . '_id', $line_response['id'] );
									}
								} else {
									update_post_meta( $v3, 'mwb_salesforce_' . $bulk_data['object'] . '_id', $line_response['id'] );
								}
							}
						}
					}
				}
			} else {

				$log_data[ $post_id ] = array_merge( $line_response, array( 'post_id' => $post_id ) );
			}
		}

		if ( ! empty( $order_item_sync_meta ) ) {
			foreach ( $order_item_sync_meta as $post_id => $sync_meta ) {
				if (Integration_With_Salesforce_Order_Handler::wps_get_cot_enabled_order_enabled() ) {
					$order = wc_get_order( $post_id );
					if ( $order ) {

						$order->update_meta_data( 'mwb_salesforce_' . $bulk_data['object'] . '_id', $sync_meta );
						$order->save();
					} else {
						update_post_meta( $post_id, 'mwb_salesforce_' . $bulk_data['object'] . '_id', $sync_meta );
					}
				} else {
					update_post_meta( $post_id, 'mwb_salesforce_' . $bulk_data['object'] . '_id', $sync_meta );
				}
			}
		}

		$this->insert_log_data( $log_data, $bulk_data );
		
		return $result_array;
	}

	/**
	 * Get request line data from line id.
	 *
	 * @param int   $line_id      Id of current request line.
	 * @param array $request_data Request bulk data.
	 *
	 * @return array  Line request data.
	 */
	public function get_request_line_data( $line_id, $request_data, $woo_object = '' ) {

		$line_request = array();
		foreach ( $request_data as $key => $request ) {
			if ( 'users' == $woo_object ) {
				
				if ( $request['user_id'] == $line_id ) {
					return $request;
				}
			} elseif ( $request['post_id'] === $line_id ) {
					return $request;
			}
		}
		return $line_request;
	}

	/**
	 * Insert log data into database.
	 *
	 * @param array $response_array Response data from API.
	 * @param array $bulk_data      Request bulk data.
	 */
	public function insert_log_data( $response_array, $bulk_data ) {

		$salesforce_object = $bulk_data['object'];
		$woo_object        = $bulk_data['woo_object'];

		foreach ( $response_array as $post_id => $response ) {
			if ( 'users' == $woo_object ) {
				$line_request = $this->get_request_line_data(
					$response['user_id'],
					$bulk_data['request'],
					'users'
				);
				$woo_id = $response['user_id'];
			} else {
				$line_request = $this->get_request_line_data(
					$response['post_id'],
					$bulk_data['request']
				);
				$woo_id = $response['post_id'];

			}
			

			$log_data = array(
				'woo_object'        => $woo_object,
				'woo_id'            => $woo_id,
				'salesforce_object' => $salesforce_object,
				'salesforce_id'     => ! empty( $response['id'] ) ? $response['id'] : '-',
				'event'             => ! empty( $bulk_data['feed_title'] ) ? $bulk_data['feed_title'] : '-',
				'request'           => wp_json_encode( $line_request ),
				'response'          => wp_json_encode( $response ),
				'time'              => time(),
			);
			mwb_woo_salesforce_insert_log( $log_data );
		}
	}

	/**
	 * Delete records from salesforce.
	 *
	 * @param array $posted_data Array of post data.
	 * @return array             Response data.
	 */
	public function delete_salesforce_record_request( $posted_data = array() ) {
		$object_type = isset( $posted_data['object_type'] ) ? $posted_data['object_type'] : false;
		$response    = array(
			'success' => false,
			'data'    => array(),
		);
		if ( ! $object_type ) {
			return $response;
		}
		$crm_api_module = Integration_With_Salesforce::get_integration_module( 'api' );
		$query          = sprintf( 'SELECT Id FROM %s LIMIT 20', $object_type );

		$query_records  = $crm_api_module->query_records( $query );
		$ids_to_deleted = array();

		if ( count( $query_records ) ) {
			foreach ( $query_records as $key => $record ) {
				$ids_to_deleted[] = $record['Id'];
			}
			$return   = $crm_api_module->delete_bulk_records( $ids_to_deleted );
			$response = array(
				'success' => true,
				'data'    => $return,
				'message' => __( 'Deleted records from Salesforce.', 'integration-with-salesforce' ),
			);
		} else {
			$response['message'] = __( 'No data found.', 'integration-with-salesforce' );
			return $response;
		}

		// $return = $crm_api_module->delete_bulk_records( $ids_to_deleted );

		// $response = array(
		// 'success' => true,
		// 'data'    => $return,
		// );
		return $response;
	}

	/**
	 * Check current if account has api enabled.
	 *
	 * @param  array $posted_data Array of posted data.
	 * @return array Response data.
	 */
	public function save_pricebook_data( $posted_data = array() ) {
		$response = array(
			'success' => false,
			'message' => __( 'Something went wrong!!', 'integration-with-salesforce' ),
		);

		$selected_pricebook = ! empty( $posted_data['pricebook'] ) ? $posted_data['pricebook'] : '';

		if ( empty( $selected_pricebook ) ) {
			return $response;
		}

		$crm_api_module = Integration_With_Salesforce::get_integration_module( 'api' );
		$pricebook_data = $crm_api_module->get_pricebook_data();

		$standard_pricebook_id = '';
		if ( is_array( $pricebook_data ) && count( $pricebook_data ) ) {
			foreach ( $pricebook_data as $key => $pricebook_item ) {
				if ( $pricebook_item['IsStandard'] ) {
					$standard_pricebook_id = $pricebook_item['Id'];
				}
			}
		}
		if ( empty( $standard_pricebook_id ) ) {
			$response['message'] = __( 'Standard Pricebook not found', 'integration-with-salesforce' );
			return $response;
		}
		update_option( 'mwb_salesforce_selected_pricebook', $selected_pricebook );
		update_option( 'mwb_salesforce_standard_pricebook', $standard_pricebook_id );

		$response = array(
			'success' => true,
			'message' => __( 'Pricebook Saved', 'integration-with-salesforce' ),
		);

		return $response;
	}

	/**
	 * Get feed ids for a group of objects.
	 *
	 * @param  array $posted_data Array of posted data.
	 * @return array Response data.
	 */
	public function get_object_group_feed_ids( $posted_data = array() ) {
		$response     = array(
			'success' => false,
			'message' => __( 'Something went wrong!!', 'integration-with-salesforce' ),
		);
		$object_group = ! empty( $posted_data['group'] ) ? $posted_data['group'] : '';

		if ( empty( $object_group ) ) {
			return $response;
		}

		$feed_module = Integration_With_Salesforce::get_integration_module( 'feed' );
		$feed_ids    = $feed_module->get_feeds_for_object_group( $object_group, 'ids' );
		if ( ! empty( $feed_ids ) ) {
			$response = array(
				'success'  => true,
				'message'  => __( 'Success', 'integration-with-salesforce' ),
				'feed_ids' => $feed_ids,
			);
		}
		return $response;
	}

	/**
	 * Check current if account has api enabled.
	 *
	 * @return array Response data.
	 */
	public function validate_api_detail() {

		$response = array(
			'success' => false,
			'code'    => 400,
			'message' => __( 'Something went wrong!! Please try another account or check with Salesforce', 'integration-with-salesforce' ),
		);

		$crm_api_module = Integration_With_Salesforce::get_integration_module( 'api' );
		$result         = $crm_api_module->get_crm_objects( true, true );
		$response       = ! empty( $result ) ? $result : $response;
		delete_option( 'mwb_salesforce_valid_api' );
		if ( isset( $response['code'] ) ) {
			if ( 200 === $response['code'] && 'OK' === $response['message'] ) {
				update_option( 'mwb_salesforce_valid_api', '1' );
				$response['success']      = true;
				$response['message']      = __( 'Validation successful !! Redirecting...', 'integration-with-salesforce' );
				$response['redirect_url'] = admin_url( 'admin.php?page=integration-with-salesforce' );
			} elseif ( 403 === $response['code'] && 'Forbidden' === $response['message'] ) {
				if ( ! empty( $response['data'] ) && ! empty( $response['data'][0] ) ) {
					$response['message'] = sprintf(
						/* translators: 1 : Error message 2: Error code */
						__(
							'%1$1s Error code : %2$2s. Please upgrade your plan or try another account.',
							'integration-with-salesforce'
						),
						$response['data'][0]['message'],
						$response['data'][0]['errorCode']
					);
				}
			}
		}
		return $response;
	}

	/**
	 * Get Content for app setup guide.
	 *
	 * @param  array $posted_data Array of posted data.
	 * @return array Response data.
	 */
	public function get_app_setup_guide_content( $posted_data = array() ) {
		$response = array(
			'success' => false,
			'message' => __( 'Something went wrong!! Please try another account or check with Salesforce', 'integration-with-salesforce' ),
		);

		$custom_app = isset( $posted_data['custom_app'] ) ? $posted_data['custom_app'] : 'no';

		$params = array(
			'callback_url'    => admin_url(),
			'setup_guide_url' => 'https://docs.woocommerce.com/document/salesforce-integration/',
			'allowed_html'    => array(
				'a'      => array(
					'href'   => array(),
					'title'  => array(),
					'target' => array(),
				),
				'strong' => array(),
			),
			'custom_app'      => $custom_app,
		);

		ob_start();
		$file_path = 'setup/setup-guide.php';
		Integration_With_Salesforce_Framework::load_template( $file_path, $params, INTEGRATION_WITH_SALESFORCE_PATH );
		$output = ob_get_contents();
		ob_end_clean();
		if ( ! empty( $output ) ) {
			$response = array(
				'success' => true,
				'content' => $output,
				'title'   => __( 'How to get the API keys?', 'integration-with-salesforce' ),
			);
		}
		return $response;
	}

	/**
	 * Delete woo meta keys.
	 *
	 * @param  array $posted_data Array of posted data.
	 * @return array Response data.
	 */
	public function reset_salesforce_woo_meta( $posted_data = array() ) {
		$response = array();
		$object   = $posted_data['object_type'];

		$meta_key = 'mwb_salesforce_' . $object . '_id';
		global $wpdb;
		$order_object_case = array( 'Order', 'Opportunity', 'Contact', 'Account', 'Case', 'Quote', 'Lead', 'OrderItem', 'OpportunityLineItem', 'QuoteLineItem' );
		if ( Integration_With_Salesforce_Order_Handler::wps_get_cot_enabled_order_enabled() && in_array( $object, $order_object_case ) ) {
			$post_meta = $wpdb->prefix . 'wc_orders_meta';
			$query     = "DELETE FROM {$post_meta} WHERE `meta_key` = '$meta_key' ";
			mwb_woo_salesforce_execute_db_query( $query );
		} else {
			$post_meta = $wpdb->prefix . 'postmeta';
			$query     = "DELETE FROM {$post_meta} WHERE `meta_key` = '$meta_key' ";
			mwb_woo_salesforce_execute_db_query( $query );
		}
		$response = array( 'success' => true );
		return $response;
	}


	/**
	 * Download api log.
	 *
	 * @return array Response array.
	 */
	public function download_api_log() {
		$filename = WC_LOG_DIR . 'mwb-woo-salesforce-' . gmdate( 'Y-m-d' ) . '.log';
		if ( file_exists( $filename ) ) {
			return array(
				'success'  => true,
				'redirect' => admin_url( '?mwb_download=api' ),
			);
		}

		return array(
			'success' => false,
		);
	}

	/**
	 * Create filter field in feed form.
	 *
	 * @param    array $posted_data   Posted data.
	 * @since    1.0.0
	 * @return   array                Response data.
	 */
	public function create_feed_filters( $posted_data ) {

		$response        = array(
			'success' => false,
			'msg'     => esc_html__( 'Somthing went wrong, Refresh and try again.', 'integration-with-salesforce' ),
		);
		$selected_object = ! empty( $posted_data['crm_selected_object'] ) ? sanitize_text_field( wp_unslash( $posted_data['crm_selected_object'] ) ) : '';
		$force           = ! empty( $posted_data['force'] ) ? sanitize_text_field( wp_unslash( $posted_data['force'] ) ) : false;

		$object_fields = $this->get_field_filter_mapping_options( $selected_object, $force );
		$filter_fields = $this->get_field_filter_options();
		return array(
			'object_fields' => $object_fields,
			'filter'        => $filter_fields,
			'success'       => true,
		);
	}

	/**
	 * Get all mapping options for a salesforce field.
	 *
	 * @param  string  $selected_object selected object.
	 * @param  boolean $force  Fetch from api.
	 * @return array           Array for field option.
	 * @since    1.0.0
	 */
	public function get_field_filter_mapping_options( $selected_object, $force = false ) {
		$feed_module   = 'Integration_With_Salesforce_Feed_Module';
		$feed_instance = $feed_module::get_instance();
		$options       = $feed_instance->get_object_filtering_field( $selected_object, $force );
		return $options;
	}

	/**
	 * Get all mapping options for a filter field.
	 *
	 * @return   array           Array for field option.
	 * @since    1.0.0
	 */
	public function get_field_filter_options() {
		$framework_class    = 'Integration_With_Salesforce_Framework';
		$framework_instance = $framework_class::get_instance();
		$options            = $framework_instance->get_filter_mapping_dataset();
		return $options;
	}

	// End of class.
}
