<?php
/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    Integration_With_Salesforce
 * @subpackage Integration_With_Salesforce/public
 */

class Integration_With_Salesforce_Public {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string $plugin_name       The name of this plugin.
	 * @param      string $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version     = $version;
	}


	/**
	 * Add dummy edit user profile hook to manage param count.
	 *
	 * @param int $user_id Id of updated user.
	 *
	 * @since 1.0.2
	 */
	public function add_wps_sf_edit_user_profile_update_hook( $user_id ) {
		if ( ! is_admin() ) {
			/**
			 * Trigger associated Feeds.
			 *
			 * @since 1.0.0
			 *
			 * @param mixed  $user_id  Woo user id.
			 * @param mixed array.
			 */
			do_action( 'wps_sf_edit_user_profile_update', $user_id, array() );
		}
	}
}
