<?php
/**
 * Provide a admin area view for the plugin
 *
 * This file is used to markup the admin-facing aspects of the plugin.
 *
 * @link       https://wpswings.com/
 * @since      1.0.0
 *
 * @package    Integration_With_Salesforce
 * @subpackage Integration_With_Salesforce/admin/partials
 */

?>
<div id="mwb-feeds" class="mwb-content-wrap mwb-tabcontent">
	<ul class="mwb-about__list">
			<?php
			foreach ( $feeds as $key => $feed ) :

				$feed_title    = $feed->post_title;
				$active        = ( 'publish' === $feed->post_status ) ? 'yes' : 'no';
				$edit_link     = get_edit_post_link( $feed->ID );
				$feed_event    = $feed_module->get_feed_data( $feed->ID, 'feed_event', '' );
				$crm_object    = $feed_module->get_feed_data( $feed->ID, 'crm_object', '' );
				$primary_field = $feed_module->get_feed_data( $feed->ID, 'primary_field', '-' );

				$default_feed = $feed_module->get_feed_data( $feed->ID, 'default_feed', '' );

				$feed_event_label = isset( $feed_options[ $feed_event ] ) ? $feed_options[ $feed_event ] : '';
				$filter_applied   = $feed_module->get_feed_data( $feed->ID, 'mwb-salesforce-enable-filters', '-' );
				?>
				<li class="mwb-about__list-item">
					<div class="mwb-about__list-item-text">
						<h3 class="mwb-about__list-item-heading">
							<?php echo esc_html( $feed_title ); ?>
						</h3>
						<p>  
							<span class="mwb-about__list-item-sub-heading">
								<?php esc_html_e( 'Active', 'integration-with-salesforce' ); ?>  
							</span>
							<span><input type="checkbox" feed_id="<?php echo esc_attr( $feed->ID ); ?>" class="mwb-crm-feed-switch mwb-crm-feed-enable" name="mwb_crm_feed_active" value="yes" <?php echo esc_attr( checked( $active, 'yes' ) ); ?> /></span>  
						</p>
						<p>
							<span class="mwb-about__list-item-sub-heading"><?php esc_html_e( 'Object', 'integration-with-salesforce' ); ?></span>
							<span><?php echo esc_html( $crm_object ); ?></span>   
						</p>
						<p>
							<span class="mwb-about__list-item-sub-heading"><?php esc_html_e( 'Event', 'integration-with-salesforce' ); ?></span>
							<span><?php echo esc_html( $feed_event_label ); ?></span> 
						</p>
						<p> 
							<span class="mwb-about__list-item-sub-heading"><?php esc_html_e( 'Primary Field', 'integration-with-salesforce' ); ?></span>
							<span><?php echo esc_html( $primary_field ); ?></span> 
						</p>
						<p>
							<span class="mwb-about__list-item-sub-heading"><?php esc_html_e( 'Conditions : ', 'integration-with-salesforce' ); ?></span>
							<span><?php echo esc_html( 'yes' !== $filter_applied ? '-' : esc_html__( 'Applied', 'integration-with-salesforce' ) ); // phpcs:ignore ?></span> 
						</p>
					</div>
					<div class="mwb-about__list-item-btn">
						<a class="mwb-edit-feed-btn"
							href="<?php echo esc_attr( $edit_link ); ?>">
							<img src="<?php echo esc_url( INTEGRATION_WITH_SALESFORCE_URL . 'assets/src/images/edit.svg' ); ?>">
						</a>
						<?php if ( ! $default_feed ) : ?>
						<a class="mwb-delete-feed-btn"
							href="#" feed_id="<?php echo esc_attr( $feed->ID ); ?>">
							<img src="<?php echo esc_url( INTEGRATION_WITH_SALESFORCE_URL . 'assets/src/images/trash.svg' ); ?>">
						</a>
						<?php endif; ?>
					</div>
				</li>
			<?php endforeach; ?>
		</ul>
		<div class="mwb-about__list-item mwb-about__list-add">            
			<div class="mwb-about__list-item-btn">
				<a href="<?php echo esc_url( admin_url( 'post-new.php?post_type=mwb_' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '_feed' ) ); ?>" class="mwb-btn mwb-btn--filled">
					<?php esc_html_e( 'Add Feeds', 'integration-with-salesforce' ); ?>
				</a>
			</div>
		</div>
</div>
