<?php
/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://wpswings.com/
 * @since      1.0.0
 *
 * @package    Integration_With_Salesforce
 * @subpackage Integration_With_Salesforce/includes
 */

if ( ! class_exists( 'Integration_With_Salesforce_Framework_Base' ) ) {
	wp_die( 'Integration_With_Salesforce_Framework_Base does not exists.' );
}

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Integration_With_Salesforce
 * @subpackage Integration_With_Salesforce/includes
 */
class Integration_With_Salesforce_Feed_Module {

	/**
	 * Feed post type name.
	 *
	 * @since    1.0.0
	 * @var      string    $feed_name    Feed post type name.
	 */
	private static $feed_name;

	/**
	 * Class instance.
	 *
	 * @since    1.0.0
	 * @var      Integration_With_Salesforce_Feed_Module    $_instance    Class instance.
	 */
	protected static $_instance = null;

	/**
	 * Main Integration_With_Salesforce_Feed_Module Instance.
	 *
	 * Ensures only one instance of Integration_With_Salesforce_Feed_Module is loaded or can be loaded.
	 *
	 * @since 1.0.0
	 *
	 * @static
	 * @return Integration_With_Salesforce_Feed_Module - Main instance.
	 */
	public static function get_instance() {

		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		self::initialize();
		return self::$_instance;
	}

	/**
	 * Custruction function.
	 */
	public function __construct() {
	}

	/**
	 * Initialize properties.
	 *
	 * @since 1.0.0
	 */
	private static function initialize() {
		self::$feed_name = 'mwb_' . Integration_With_Salesforce::get_current_crm_name( 'slug' ) . '_feed';
	}

	/**
	 * Get feed name.
	 *
	 * @since 1.0.0
	 */
	private function get_feed_name() {
		return self::$feed_name;
	}

	/**
	 * Register custom post types
	 */
	public function register_plugin_cpt() {

		// Set UI labels for Custom Post Type.
		$labels = array(
			'name'               => _x( 'Feeds', 'Post Type General Name', 'integration-with-salesforce' ),
			'singular_name'      => _x( 'Feed', 'Post Type Singular Name', 'integration-with-salesforce' ),
			'menu_name'          => _x( 'Crm Feeds', 'Admin menu name', 'integration-with-salesforce' ),
			'parent_item_colon'  => __( 'Parent Feed', 'integration-with-salesforce' ),
			'all_items'          => __( 'All Feeds', 'integration-with-salesforce' ),
			'view_item'          => __( 'View Feed', 'integration-with-salesforce' ),
			'add_new_item'       => __( 'Add New Feed', 'integration-with-salesforce' ),
			'add_new'            => __( 'Add New', 'integration-with-salesforce' ),
			'edit_item'          => __( 'Edit Feed', 'integration-with-salesforce' ),
			'update_item'        => __( 'Update Feed', 'integration-with-salesforce' ),
			'search_items'       => __( 'Search Feed', 'integration-with-salesforce' ),
			'not_found'          => __( 'Not Found', 'integration-with-salesforce' ),
			'not_found_in_trash' => __( 'Not found in Trash', 'integration-with-salesforce' ),
		);

		$args = array(
			'label'               => __( 'Feeds', 'integration-with-salesforce' ),
			'description'         => __( 'Feeds for crm', 'integration-with-salesforce' ),
			'labels'              => $labels,
			'supports'            => array( 'title' ),
			'hierarchical'        => false,
			'public'              => true,
			'menu_position'       => 5,
			'can_export'          => true,
			'has_archive'         => true,
			'show_in_rest'        => false,
			'exclude_from_search' => false,
			'publicly_queryable'  => false,
			'show_in_menu'        => false,
			'show_in_nav_menus'   => false,
			'show_in_admin_bar'   => false,
		);

		register_post_type( $this->get_feed_name(), $args );
	}

	/**
	 * Add meta boxes for feeds
	 */
	public function add_feed_meta_boxes() {

		$screenid = Integration_With_Salesforce_Order_Handler::wps_get_shop_order_screen_id();

		add_meta_box(
			'crm-feed-meta-box',
			__( 'Feed Details', 'integration-with-salesforce' ),
			array( $this, 'crm_feed_meta_box_data' ),
			$this->get_feed_name(),
			'normal',
			'default'
		);

		add_meta_box(
			'mwb_salesforce_feeds_condition_meta_box',
			__( 'Conditional Statements', 'integration-with-salesforce' ),
			array( $this, 'feeds_mwb_condition_render' ),
			$this->get_feed_name(),
			'normal',
			'default'
		);

		add_meta_box(
			'mwb-salesforce-order-meta-box',
			__( 'Salesforce Manual Sync', 'integration-with-salesforce' ),
			array( $this, 'shop_order_meta_box_data' ),
			$screenid,
			'side',
			'low'
		);

		add_meta_box(
			'mwb-salesforce-order-feed-meta-box',
			__( 'Salesforce Feeds', 'integration-with-salesforce' ),
			array( $this, 'shop_order_salesforce_feed_meta_box' ),
			$screenid,
			'side',
			'low'
		);

		add_meta_box(
			'mwb-salesforce-product-meta-box',
			__( 'Salesforce Manual Sync', 'integration-with-salesforce' ),
			array( $this, 'shop_order_meta_box_data' ),
			'product',
			'side',
			'low'
		);

		add_meta_box(
			'mwb-salesforce-order-feed-meta-box',
			__( 'Salesforce Feeds', 'integration-with-salesforce' ),
			array( $this, 'shop_order_salesforce_feed_meta_box' ),
			'product',
			'side',
			'low'
		);
	}

	/**
	 * Render feed meta box.
	 */
	public function crm_feed_meta_box_data() {
		global $post;
		$param                     = array();
		$params['objects']         = Integration_With_Salesforce_Framework::get_available_crm_objects();
		$params['selected_object'] = $this->get_feed_data( $post->ID, 'crm_object', '' );
		$this->render_meta_box_data( 'select-object', $params );
	}

	/**
	 * Check if object has a property which support status mapping.
	 *
	 * @param  string $object Name of object.
	 * @return array          Array of fields which supports status mapping.
	 */
	public function if_object_support_status_mapping( $object ) {
		$supported_objects = array(
			'Opportunity' => 'StageName',
			'Order'       => 'Status',
			'Quote'       => 'Status',
		);
		/**
		 * Filters the value of Salesforce Supported Objects.
		 *
		 * @since 1.0.3
		 *
		 * @param mixed   $supported_objects  Woo order status.
		 */
		$supported_objects = apply_filters( 'mwb_woo_salesforce_status_supported_objects', $supported_objects );
		if ( isset( $supported_objects[ $object ] ) && ! empty( $supported_objects[ $object ] ) ) {
			return $supported_objects[ $object ];
		}
		return false;
	}

	/**
	 * Check if object has a property which support pricebook mapping.
	 *
	 * @param  string $object Name of object.
	 * @return array          Array of fields which supports pricebook mapping.
	 */
	public function if_object_support_pricebook_mapping( $object ) {

		$supported_objects = array(
			'PricebookEntry' => 'Pricebook2Id',
			'Order'          => 'Pricebook2Id',
			'Quote'          => 'Pricebook2Id',
		);
		if ( isset( $supported_objects[ $object ] ) && ! empty( $supported_objects[ $object ] ) ) {
			return $supported_objects[ $object ];
		}
		return false;
	}

	/**
	 * Create meta box section html via ajax.
	 *
	 * @param  string $meta_box Meta box key.
	 * @param  array  $params   Required params for meta box.
	 * @return string           Meta box section html.
	 */
	public function do_ajax_render( $meta_box, $params ) {

		if ( '' == $meta_box ) {
			return;
		}
		ob_start();
		$this->render_meta_box_data( $meta_box, $params );
		$output = ob_get_contents();
		ob_end_clean();
		return $output;
	}

	/**
	 * Render meta box.
	 *
	 * @param  string $meta_box Meta box key.
	 * @param  array  $params   Required params for meta box.
	 */
	private function render_meta_box_data( $meta_box, $params ) {

		if ( '' == $meta_box ) {
			return;
		}
		$file_path = 'woo-crm-fw/templates/meta-boxes/' . $meta_box . '.php';
		Integration_With_Salesforce_Framework::load_template( $file_path, $params, INTEGRATION_WITH_SALESFORCE_PATH );
	}

	/**
	 * Save meta box data for feed custom post type
	 *
	 * @param  int $post_id Id of the cpt.
	 */
	public function save_metabox_data( $post_id ) {

		if ( ! isset( $_POST['_wpnonce'] ) ) {
			return;
		}

		if ( ! isset( $_POST['meta_box_nonce'] ) ) {
			return;
		}

		if ( ! wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['meta_box_nonce'] ) ), 'meta_box_nonce' ) ) {
			return;
		}

		if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
			;
			return;
		}

		if ( ! current_user_can( 'edit_post', $post_id ) ) {
			return;
		}

		if ( isset( $_POST['post_type'] ) && $this->get_feed_name() === $_POST['post_type'] ) {

			$crm_object          = isset( $_POST['crm_object'] ) ? sanitize_text_field( wp_unslash( $_POST['crm_object'] ) ) : '';
			$crm_field_arr       = isset( $_POST['crm_field'] ) ? map_deep( wp_unslash( $_POST['crm_field'] ), 'sanitize_text_field' ) : array();
			$field_type_arr      = isset( $_POST['field_type'] ) ? map_deep( wp_unslash( $_POST['field_type'] ), 'sanitize_text_field' ) : array();
			$field_value_arr     = isset( $_POST['field_value'] ) ? map_deep( wp_unslash( $_POST['field_value'] ), 'sanitize_text_field' ) : array();
			$custom_value_arr    = isset( $_POST['custom_value'] ) ? map_deep( wp_unslash( $_POST['custom_value'] ), 'sanitize_text_field' ) : array();
			$custom_field_arr    = isset( $_POST['custom_field'] ) ? map_deep( wp_unslash( $_POST['custom_field'] ), 'sanitize_text_field' ) : array();
			$woo_status          = isset( $_POST['woo_status'] ) ? map_deep( wp_unslash( $_POST['woo_status'] ), 'sanitize_text_field' ) : array();
			$crm_picklist_value  = isset( $_POST['crm_picklist_value'] ) ? map_deep( wp_unslash( $_POST['crm_picklist_value'] ), 'sanitize_text_field' ) : array();
			$use_status_mapping  = isset( $_POST['use_status_mapping'] ) ? map_deep( wp_unslash( $_POST['use_status_mapping'] ), 'sanitize_text_field' ) : array();
			$primary_field       = isset( $_POST['primary_field'] ) ? sanitize_text_field( wp_unslash( $_POST['primary_field'] ) ) : '';
			$feed_event          = isset( $_POST['feed_event'] ) ? sanitize_text_field( wp_unslash( $_POST['feed_event'] ) ) : '';
			$pricebook_id        = isset( $_POST['selected_pricebook'] ) ? sanitize_text_field( wp_unslash( $_POST['selected_pricebook'] ) ) : '';
			$enable_filters      = isset( $_POST['enable_add_condition'] ) ? sanitize_text_field( wp_unslash( $_POST['enable_add_condition'] ) ) : '';
			$condition           = isset( $_POST['condition'] ) ? map_deep( wp_unslash( $_POST['condition'] ), 'sanitize_text_field' ) : array();
			$sync_parent_product = isset( $_POST['sync_parent_product'] ) ? sanitize_text_field( wp_unslash( $_POST['sync_parent_product'] ) ) : 'no';
			$sync_person_account = isset( $_POST['sync_person_account'] ) ? sanitize_text_field( wp_unslash( $_POST['sync_person_account'] ) ) : 'no';
			$mapping_data        = array();
			foreach ( $crm_field_arr as $key => $field ) {
				$field_key                  = $field;
				$mapping_data[ $field_key ] = array(
					'field_type'         => $field_type_arr[ $key ],
					'field_value'        => $field_value_arr[ $key ],
					'custom_value'       => $custom_value_arr[ $key ],
					'custom_field'       => $custom_field_arr[ $key ],
					'use_status_mapping' => isset( $use_status_mapping[ $field_key ] ) ? 'yes' : 'no',
				);
			}

			$status_mapping = array();
			if ( count( $woo_status ) == count( $crm_picklist_value ) ) {
				foreach ( $woo_status as $key => $value ) {
					$status_mapping[ $value ] = $crm_picklist_value[ $key ];
				}
			}

			if ( 'Product2' === $crm_object && '' !== $pricebook_id ) {
				update_option( 'mwb_salesforce_selected_pricebook', $pricebook_id );
				update_post_meta( $post_id, 'sync_parent_product', $sync_parent_product );
			}

			update_post_meta( $post_id, 'sync_person_account', $sync_person_account );

			update_post_meta( $post_id, 'crm_object', $crm_object );
			update_post_meta( $post_id, 'mapping_data', $mapping_data );
			update_post_meta( $post_id, 'status_mapping', $status_mapping );
			update_post_meta( $post_id, 'primary_field', $primary_field );
			update_post_meta( $post_id, 'feed_event', $feed_event );
			update_post_meta( $post_id, 'selected_pricebook', $pricebook_id );
			update_post_meta( $post_id, 'mwb-salesforce-enable-filters', $enable_filters );
			update_post_meta( $post_id, 'mwb-salesforce-condition-field', $condition );
		}
	}

	/**
	 * Get woocommerce order statuses.
	 *
	 * @return array Fitlered status array.
	 */
	public function get_woo_order_statuses() {
		$order_statuses = wc_get_order_statuses();
		/**
		 * Filters the value of Salesforce Order Status.
		 *
		 * @since 1.0.3
		 *
		 * @param mixed   $order_statuses  Woo order status.
		 */
		return apply_filters( 'mwb_woo_crm_order_statuses', $order_statuses );
	}

	/**
	 * Get feed data based on meta key.
	 *
	 * @param int|bool $post_id Feed post id.
	 * @param string   $key     Meta key.
	 * @param mix      $default Default data.
	 * @param string   $selected_object Selected object name.
	 *
	 * @return mix  Meta data.
	 */
	public function get_feed_data( $post_id, $key, $default = '', $selected_object = '' ) {

		if ( ! $post_id ) {
			$method = 'get_default_' . $key;
			if ( method_exists( $this, $method ) && '' != $selected_object ) {
				$default = $this->$method( $selected_object );
			}
			return $default;
		}
		$feed_data = get_post_meta( $post_id, $key, true );
		$feed_data = ! empty( $feed_data ) ? $feed_data : $default;
		return $feed_data;
	}

	/**
	 * Get feed trigger event options.
	 *
	 * @param  array $order_statuses WooCommerce Order Statuses.
	 * @param  int   $feed_id        Feed id.
	 * @return array Event options.
	 */
	public function get_feed_event_options( $order_statuses = array(), $feed_id = false ) {

		if ( empty( $order_statuses ) ) {
			$order_statuses = $this->get_woo_order_statuses();
		}

		$admin_instance       = new Integration_With_Salesforce_Admin( 'Salesforce Integration for WooCommerce', '1.2.1' );
		$options              = array();
		$options['new-order'] = __( 'New order is created', 'integration-with-salesforce' );
		foreach ( $order_statuses as $key => $value ) {
			// translators: Order status label.
			$options[ $key ] = sprintf( __( 'Order Status Change to %s', 'integration-with-salesforce' ), $value );
		}
		$options['status_change']         = __( 'Order Status Change', 'integration-with-salesforce' );
		$options['product_update_create'] = __( 'Product Updated/Created', 'integration-with-salesforce' );
		$options['send_manually']         = __( 'Send Manually', 'integration-with-salesforce' );

		if ( $admin_instance->wps_is_woocommerce_subscription_plugin_active() ) {
			$options['create_subscription']         = __( 'Subscription is created', 'integration-with-salesforce' );
			$options['subscription_status_changed'] = __( 'Subscription Status Change', 'integration-with-salesforce' );
			$options['subscription_updated']        = __( 'Subscription Updated', 'integration-with-salesforce' );
		}

		if ( $admin_instance->wps_is_woocommerce_membership_plugin_active() ) {
			$options['membership_saved']         = __( 'Membership is saved', 'integration-with-salesforce' );
			// $options['membership_created_when_adding_new_user_as_member']         = __( 'Membership created when adding new user as member', 'integration-with-salesforce' );
			// $options['membership_created_when_adding_existing_user_as_member']         = __( 'Membership adding existing user', 'integration-with-salesforce' );
		}

		$options['user_update_create'] = __( 'WP User Updated/Created', 'integration-with-salesforce' );

		$available_feeds = $this->get_available_feeds_for_mapping();

		foreach ( $available_feeds as $f_id => $feed_title ) {
			if ( intval( $f_id ) === intval( $feed_id ) ) {
				continue;
			}
			// translators: feed title.
			$options[ $f_id ] = sprintf( __( 'After %s', 'integration-with-salesforce' ), $feed_title );
		}

		return $options;
	}

	/**
	 * Get default fields data to create initially.
	 *
	 * @return array Default feeds array.
	 */
	public function get_default_feeds() {

		$mapping_json    = mwb_woo_salesforce_get_mapping_json_data();
		$mapping_data    = json_decode( $mapping_json, ARRAY_A );
		$default_objects = array_keys( $mapping_data );

		foreach ( $default_objects as $key => $object ) {
			// translators: Object Name.
			$title           = sprintf( __( '%s Feed', 'integration-with-salesforce' ), $object );
			$dependency      = $this->get_feed_dependencies( $object );
			$default_feeds[] = compact( 'title', 'object', 'dependency' );
		}
		/**
		 * Filters the value of Salesforce Default Feeds.
		 *
		 * @since 1.0.3
		 *
		 * @param mixed   $default_feeds  Default Feeds.
		 */
		return apply_filters( 'mwb_woo_salesforce_default_feeds', $default_feeds );
	}

	/**
	 * Get meta data related to a feed object.
	 *
	 * @param  string $feed_object Feed Object.
	 * @return array               Meta data related to object.
	 */
	public function get_feed_meta_data( $feed_object ) {

		$meta_input = array(
			'crm_object'    => $feed_object,
			'mapping_data'  => $this->get_default_mapping_data( $feed_object ),
			'feed_event'    => $this->get_default_feed_event( $feed_object ),
			'primary_field' => $this->get_default_primary_field( $feed_object ),
			'default_feed'  => true,
		);

		$status_mapping_field = $this->if_object_support_status_mapping( $feed_object );
		if ( ! empty( $status_mapping_field ) ) {
			$meta_input['status_mapping'] = $this->get_default_status_mapping( $feed_object, $status_mapping_field );
		}

		if ( $this->if_object_support_pricebook_mapping( $feed_object ) || 'Product2' === $feed_object ) {
			$meta_input['selected_pricebook'] = $this->get_default_selected_pricebook( $feed_object );
		}
		return $meta_input;
	}

	/**
	 * Get default pricebook id.
	 *
	 * @param  string $crm_object Object Name.
	 * @return string            Pricebook id.
	 */
	public function get_default_selected_pricebook( $crm_object ) {
		// Get pricebook id from options if already saved.
		$pricebook_id = get_option( 'mwb_salesforce_selected_pricebook', false );
		if ( ! $pricebook_id ) {
			// Pricebook is not yet selected.
			// Fetch standard pricebook details and save as default.
			$crm_api_module = Integration_With_Salesforce::get_integration_module( 'api' );
			$pricebook_data = $crm_api_module->get_pricebook_data( false );
			// Got the pricebook data.
			// Now select the standard pricebook.
			if ( ! empty( $pricebook_data ) ) {
				foreach ( $pricebook_data as $key => $data ) {
					if ( $data['IsStandard'] ) {
						$pricebook_id = $data['Id'];
						break;
					}
				}
			}
		}
		return $pricebook_id;
	}

	/**
	 * Get default mapping of picklist values with woo order statuses.
	 *
	 * @param  string $crm_object     CRM object name.
	 * @param  string $object_field   Object field name.
	 * @param  array  $order_statuses Woo order statuses.
	 * @return Array                  Default mapping array.
	 */
	public function get_default_status_mapping( $crm_object, $object_field = '', $order_statuses = array() ) {

		$status_mapping = array();
		if ( empty( $order_statuses ) ) {
			$order_statuses = wc_get_order_statuses();
		}
		if ( 'Opportunity' === $crm_object && 'StageName' === $object_field ) {
			foreach ( $order_statuses as $key => $value ) {
				switch ( $key ) {
					case 'wc-pending':
						$status_mapping[ $key ] = 'Needs Analysis';
						break;
					case 'wc-processing':
					case 'wc-completed':
						$status_mapping[ $key ] = 'Closed Won';
						break;
					case 'wc-on-hold':
						$status_mapping[ $key ] = 'Negotiation/Review';
						break;
					case 'wc-cancelled':
					case 'wc-refunded':
					case 'wc-failed':
						$status_mapping[ $key ] = 'Closed Lost';
						break;
					default:
						$status_mapping[ $key ] = 'Prospecting';
						break;
				}
			}
		} elseif ( ( 'Order' === $crm_object || 'Quote' === $crm_object ) && 'Status' === $object_field ) {
			foreach ( $order_statuses as $key => $value ) {
				$status_mapping[ $key ] = 'Draft';
			}
		}
		return $status_mapping;
	}

	/**
	 * Get dependent field and accosiated objects.
	 *
	 * @param  string $crm_object CRM object name.
	 * @return array              Array dependencies.
	 */
	public function get_feed_dependencies( $crm_object ) {
		$dependency = array();
		switch ( $crm_object ) {
			case 'Contact':
			case 'Opportunity':
				$dependency = array( 'AccountId' => 'Account' );
				break;
			case 'Order':
				$dependency = array(
					'AccountId'    => 'Account',
					'Pricebook2Id' => 'Pricebook2',
				);
				break;
			case 'PricebookEntry':
				$dependency = array(
					'Product2Id'   => 'Product2',
					'Pricebook2Id' => 'Pricebook2',
				);
				break;
			case 'OrderItem':
				$dependency = array(
					'OrderId'          => 'Order',
					'PricebookEntryId' => 'PricebookEntry',
				);
				break;
			case 'OpportunityLineItem':
				$dependency = array(
					'OpportunityId'    => 'Opportunity',
					'PricebookEntryId' => 'PricebookEntry',
				);
				break;
			default:
				$dependency = array();
				break;
		}
		return $dependency;
	}

	/**
	 * Get default mapping and data.
	 *
	 * @param  string $crm_object CRM object name.
	 * @return array              Default mapping array.
	 */
	public function get_default_mapping_data( $crm_object ) {

		$dependent_fields = $this->get_feed_dependencies( $crm_object );

		$mapping_json  = mwb_woo_salesforce_get_mapping_json_data();
		$mapping_array = json_decode( $mapping_json, ARRAY_A );
		$mapping_data  = isset( $mapping_array[ $crm_object ] ) ? $mapping_array[ $crm_object ] : array();
		if ( ! empty( $dependent_fields ) ) {
			foreach ( $dependent_fields as $field => $object ) {

				$object_feed_id                        = get_option( 'mwb_woo_salesforce_' . $object . '_feed_id', '' );
				$mapping_data[ $field ]['field_type']  = 'standard_field';
				$mapping_data[ $field ]['field_value'] = 'feeds_' . $object_feed_id;

				if ( 'Pricebook2Id' == $field ) {
					$mapping_data[ $field ]['field_value'] = get_option( 'mwb_salesforce_selected_pricebook', '' );
				}
			}
		}
		return $mapping_data;
	}

	/**
	 * Get event to trigger the feed.
	 *
	 * @param  string $object Object Name.
	 * @return string         Event key.
	 */
	public function get_default_feed_event( $object ) {
		$event = '';
		switch ( $object ) {
			case 'Account':
				$event = 'new-order';
				break;
			case 'Contact':
				$event = 'after_Account';
				break;
			case 'Product2':
				$event = 'product_update_create';
				break;
			case 'PricebookEntry':
				$event = 'after_Product2';
				break;
			case 'Opportunity':
				$event = 'status_change';
				break;
			case 'Order':
				$event = 'after_Opportunity';
				break;
			case 'OrderItem':
				$event = 'after_Opportunity';
				break;
			default:
				$event = 'send_manually';
				break;
		}

		if ( false !== strpos( $event, 'after_' ) ) {
			$feed_object    = str_replace( 'after_', '', $event );
			$object_feed_id = get_option( 'mwb_woo_salesforce_' . $feed_object . '_feed_id', '' );
			if ( ! empty( $object_feed_id ) ) {
				$event = $object_feed_id;
			}
		}

		return $event;
	}

	/**
	 * Get default primary field for an object
	 *
	 * @param  string $object Object Name.
	 * @return string         Primary Field Name.
	 */
	public function get_default_primary_field( $object ) {

		$default_primary = '';
		switch ( $object ) {
			case 'Contact':
				$default_primary = 'Email';
				break;
			case 'Order':
				$default_primary = 'Name';
				break;
			case 'Account':
				$default_primary = 'Name';
				break;
			case 'Opportunity':
				$default_primary = 'Name';
				break;
			case 'Product2':
				$default_primary = 'Name';
				break;
		}
		return $default_primary;
	}

	/**
	 * Create feed post.
	 *
	 * @param  string $feed_title  Title of the feed.
	 * @param  string $feed_object Object of the feed.
	 * @return int                  Feed id.
	 */
	public function create_feed_post( $feed_title, $feed_object ) {
		$args = array(
			'post_type'   => $this->get_feed_name(),
			'post_status' => 'publish',
			'post_title'  => $feed_title,
			'meta_input'  => $this->get_feed_meta_data( $feed_object ),
		);

		$feed_id = wp_insert_post( $args );
		return $feed_id;
	}

	/**
	 * Check is feed post exists.
	 *
	 * @param  int $feed_id Feed id.
	 * @return bool
	 */
	public function is_valid_feed_post( $feed_id ) {
		$post   = get_post( $feed_id );
		$return = false;

		if ( ! empty( $post )
		&& 'publish' === $post->post_status
		&& $this->get_feed_name() === $post->post_type ) {
			$return = $feed_id;
		}

		return $return;
	}

	/**
	 * Get available feeds for mapping.
	 *
	 * @return array Feed list.
	 */
	public function get_available_feeds_for_mapping() {
		$all_feeds = $this->get_available_feeds();
		if ( ! $all_feeds ) {
			return array();
		}
		$feed_list = array();
		foreach ( $all_feeds as $key => $feed ) {
			$feed_list[ $feed->ID ] = $feed->post_title;
		}
		return $feed_list;
	}

	/**
	 * Get available feeds.
	 *
	 * @param array $status Array of status.
	 */
	public function get_available_feeds( $status = array() ) {

		$status = empty( $status ) ? 'publish' : $status;
		$args   = array(
			'post_type'   => $this->get_feed_name(),
			'post_status' => $status,
			'numberposts' => -1,
			'orderby'     => 'id',
			'order'       => 'ASC',
		);
		$feeds  = get_posts( $args );
		return $feeds;
	}


	/**
	 * Add shop order feeds meta box.
	 *
	 * @param mixed $post_or_order_object  Post object or WC instance.
	 * @return void
	 */
	public function shop_order_salesforce_feed_meta_box( $post_or_order_object ) {
		if ( $post_or_order_object instanceof WC_Order ) {
			$post_id   = $post_or_order_object->get_id();
			$post_type = $post_or_order_object->get_type();
		} else {
			$post_id   = $post_or_order_object->ID;
			$post_type = $post_or_order_object->post_type;
		}
		$feed_list     = array();
		$feeds         = $this->get_available_feeds_for_mapping();
		$variation_ids = array();
		$item_type     = ( 'product' === $post_type ) ? 'Product id' : 'Order id';
		if ( 'product' === $post_type ) {
			$product = wc_get_product( $post_id );
			if ( 'variable' === $product->get_type() || 'variable-subscription' === $product->get_type() ) {
				$variations = $product->get_available_variations();
				if ( ! empty( $variations ) && is_array( $variations ) ) {
					foreach ( $variations as $key => $variation ) {
						$variation_ids[] = $variation['variation_id'];
					}
				}
			}
		}

		if ( count( $feeds ) ) {
			foreach ( $feeds as $feed_id => $feed_title ) {
				$crm_object = $this->get_feed_data( $feed_id, 'crm_object' );
				if ( count( $variation_ids ) ) {
					$product_feed_id = get_option( 'mwb_woo_salesforce_Product2_feed_id', '' );
					if ( ! empty( $product_feed_id ) ) {
						$is_default_feed = $this->get_feed_data( $product_feed_id, 'default_feed', '' );
						if ( $is_default_feed ) {
							$sync_parent_product = $this->get_feed_data( $product_feed_id, 'sync_parent_product', 'no' );
							if ( 'yes' === $sync_parent_product ) {
								$reference_id = wps_salesforce_get_meta_data( $post_id, 'mwb_salesforce_' . $crm_object . '_id', true );
								if ( ! empty( $reference_id ) ) {
									$feed_list[] = compact( 'feed_title', 'reference_id', 'crm_object' );
								}
							}
						}
					}
					foreach ( $variation_ids as $vk => $vid ) {
						$reference_id = wps_salesforce_get_meta_data( $vid, 'mwb_salesforce_' . $crm_object . '_id', true );
						if ( ! empty( $reference_id ) ) {
							$item_id     = $vid;
							$feed_list[] = compact( 'feed_title', 'reference_id', 'item_type', 'item_id', 'crm_object' );
						}
					}
				} else {

					$object_type = Integration_With_Salesforce_Order_Handler::wps_get_object_type( $post_id );

					if ( Integration_With_Salesforce_Order_Handler::wps_get_cot_enabled_order_enabled() ) {
						if ( 'product' == $object_type ) {

							$reference_data = get_post_meta( $post_id, 'mwb_salesforce_' . $crm_object . '_id', true );
						} else {
							$order = wc_get_order( $post_id );

							if ( $order ) {

								$reference_data = $order->get_meta( 'mwb_salesforce_' . $crm_object . '_id' );
							} else {
								$reference_data = wps_salesforce_get_meta_data( $post_id, 'mwb_salesforce_' . $crm_object . '_id', true );
							}
						}
					} else {
						$reference_data = wps_salesforce_get_meta_data( $post_id, 'mwb_salesforce_' . $crm_object . '_id', true );
					}

					if ( ! empty( $reference_data ) ) {
						if ( is_array( $reference_data ) ) {
							foreach ( $reference_data as $k => $v ) {
								$reference_id = $v['id'];
								$item_id      = $post_id;
								$feed_list[]  = compact( 'feed_title', 'reference_id', 'crm_object' );
							}
						} else {
							$reference_id = $reference_data;
							$item_id      = $post_id;
							$feed_list[]  = compact( 'feed_title', 'reference_id', 'crm_object' );
						}
					}
				}
			}
		}

		$crm_api_module = Integration_With_Salesforce::get_integration_module( 'api' );
		$instance_url   = $crm_api_module->get_instance_url();

		$this->render_meta_box_data( 'feed-list-meta-box', compact( 'feed_list', 'instance_url' ) );
	}

	/**
	 * Add shop order manual sync meta box.
	 *
	 * @param mixed $post_or_order_object  Post object or WC instance.
	 * @return void
	 */
	public function shop_order_meta_box_data( $post_or_order_object ) {
		if ( $post_or_order_object instanceof WC_Order ) {
			$post_order_id = $post_or_order_object->get_id();
		} else {
			$post_order_id = $post_or_order_object->ID;
		}
		// $post_order_id  = $post_or_order_object->ID;

		$params = array();
		$feeds  = $this->get_available_feeds_for_mapping();

		$object_type = Integration_With_Salesforce_Order_Handler::wps_get_object_type( $post_order_id );

		foreach ( $feeds as $feed_id => $feed_title ) {
			$crm_object = $this->get_feed_data( $feed_id, 'crm_object' );
			if ( $object_type !== $this->get_feed_woo_object( $crm_object, $feed_id ) ) {
				unset( $feeds[ $feed_id ] );
			}
		}

		$this->render_meta_box_data( 'manual-sync-meta-box', compact( 'feeds', 'post_order_id' ) );
	}

	/**
	 * Check for default feed post.
	 *
	 * @param  string $crm_object   CRM Object.
	 * @return int/bool             Feed id or False.
	 */
	public function check_for_feed_post( $crm_object ) {

		$query_args = array(
			array(
				'key'     => 'crm_object',
				'value'   => $crm_object,
				'compare' => '=',
			),
			array(
				'key'     => 'default_feed',
				'value'   => true,
				'compare' => '=',
			),
		);

		$args = array(
			'post_type'   => $this->get_feed_name(),
			'post_status' => 'publish',
			'meta_query'  => $query_args, //phpcs:ignore
		);

		$feeds = get_posts( $args );

		$feed_id = ! empty( $feeds ) ? $feeds[0]->ID : false;
		return $feed_id;
	}

	/**
	 * Get feeds attached after a particular feed.
	 *
	 * @param int $feed_id Feed id.
	 * @return array       Array of feed ids.
	 */
	public function get_associated_feeds( $feed_id ) {

		$feeds = array();
		$args  = array(
			'post_type'   => $this->get_feed_name(),
			'post_status' => 'publish',
			'numberposts' => -1,
			'orderby'     => 'id',
			'order'       => 'ASC',
			'meta_query'  => array(
				array(
					'key'     => 'feed_event',
					'value'   => $feed_id,
					'compare' => '=',
				),
			),
			'fields'      => 'ids',
		);
		$feeds = get_posts( $args );
		return $feeds;
	}

	/**
	 * Get associated feed ids for a group.
	 *
	 * @param  string $object_group Name of object group.
	 * @param  string $return_type  Return type.
	 * @return array  Array feed ids.
	 */
	public function get_feeds_for_object_group( $object_group, $return_type = '' ) {

		$feeds = array();

		$crm_objects = $this->get_object_from_group_name( $object_group );

		if ( count( $crm_objects ) > 0 ) {
			foreach ( $crm_objects as $key => $object ) {
				$feeds[] = $this->check_for_feed_post( $object );
			}
		}

		return $feeds;
	}

	/**
	 * Get associated crm objects in a group.
	 *
	 * @param  string $object_group Name of object group.
	 * @return array  Array of crm objects.
	 */
	public function get_object_from_group_name( $object_group ) {

		$object_groups = array(
			'Product' => array( 'Product2', 'PricebookEntry' ),
			'Contact' => array( 'Account', 'Contact' ),
			'Order'   => array( 'Opportunity', 'Order', 'OrderItem' ),
		);

		return isset( $object_groups[ $object_group ] ) ? $object_groups[ $object_group ] : array();
	}

	/**
	 * Get feed associated woocommerce object.
	 *
	 * @param  string $crm_object Name of object.
	 * @param  string $feed_id    Feed id.
	 * @return string            Woo Object key.
	 */
	private function get_feed_woo_object( $crm_object, $feed_id ) {
		$woo_object     = '';
		$request_module = Integration_With_Salesforce::get_integration_module( 'request' );
		switch ( $crm_object ) {
			case 'Account':
			case 'Contact':
			case 'Lead':
				$woo_object = $request_module->get_woo_object_based_on_event( $feed_id );
				break;
			case 'Order':
			case 'Opportunity':
			case 'OrderItem':
			case 'Case':
			case 'OpportunityLineItem':
			case 'Quote':
			case 'QuoteLineItem':
				$woo_object = 'shop_order';
				break;
			case 'Product2':
			case 'PricebookEntry':
				$woo_object = 'product';
				break;
			default:
				$woo_object = $request_module->get_woo_object_based_on_field_mapping( $feed_id );
				break;
		}
		return $woo_object;
	}

	/**
	 * Post type feeds conditional filter metabox.
	 *
	 * @since     1.0.0
	 * @return    void
	 */
	public function feeds_mwb_condition_render() {
		global $post;
		$params                  = array();
		$selected_object         = $this->get_feed_data( $post->ID, 'crm_object', '' );
		$params['enable_filter'] = $this->get_feed_data( $post->ID, 'mwb-salesforce-enable-filters', '' );
		$params['condition']     = $this->get_feed_data( $post->ID, 'mwb-salesforce-condition-field', array() );
		$params['fields']        = $this->get_object_filtering_field( $selected_object, false );
		$this->render_meta_box_data( 'opt-in-condition', compact( 'params' ) );
	}

	/**
	 * Get selected object api fields for filtering.
	 *
	 * @param  string  $selected_object   selected object.
	 * @param  boolean $force  Fetch from api.
	 * @since 1.0.0
	 *
	 * @return array - Current CRM to Woo mapping.
	 */
	public function get_object_filtering_field( $selected_object = '', $force = false ) {
		$formatted_dataset   = array();
		$crm_integration_api = new Integration_With_Salesforce_Api();
		$object_fields       = $crm_integration_api->get_object_fields( $selected_object, $force );
		if ( empty( $object_fields ) || ! is_array( $object_fields ) ) {
			return;
		}
		$crm_fields = array();
		foreach ( $object_fields as $key => $fields_data ) {
			if ( ! $fields_data['updateable'] ) {
				continue;
			}
			array_push( $crm_fields, $fields_data );
		}
		$api_name                              = array_column( $crm_fields, 'label', 'name' );
		$formatted_dataset[ $selected_object ] = $api_name;
		return $formatted_dataset;
	}

	/**
	 * Function to add user on user edit page.
	 *
	 * @param object $user user object.
	 * @return void
	 */
	public function add_feed_data_to_user_page( $user ) {

		$feeds = $this->get_available_feeds_for_mapping();
		?>
		<h3><?php echo esc_html( 'Salesforce Feeds' ); ?></h3>
		<?php
		foreach ( $feeds as $feed_id => $feed_title ) {
			$crm_object = $this->get_feed_data( $feed_id, 'crm_object' );

			if ( 'users' !== $this->get_feed_woo_object( $crm_object, $feed_id ) ) {
				unset( $feeds[ $feed_id ] );
			}
		}

		if ( ! empty( $feeds ) ) {

			?>
			<table class="form-table">
				<?php
				foreach ( $feeds as $feed_id => $feed_title ) {
					$crm_object     = $this->get_feed_data( $feed_id, 'crm_object' );
					$reference_data = get_user_meta( $user->ID, 'mwb_salesforce_' . $crm_object . '_id', true );
					if ( ! empty( $reference_data ) ) {
						?>
		
					<tr>
						<th><label for="feed_title"><?php esc_html_e( $feed_title ); ?></label></th>
						<td>
						<input type="text" name="<?php echo esc_attr( $feed_title ); ?>" disabled id="<?php echo esc_attr( $feed_id ); ?>" class="item-id-link" target="_blank" value="<?php echo esc_html( $reference_data ); ?>"	>
						</td>
					</tr>
						<?php
					}
					?>
				</table>
					<?php
				}
		}
	}
}
